# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.slo_list_widget_query import SLOListWidgetQuery
    from datadog_api_client.v1.model.slo_list_widget_request_type import SLOListWidgetRequestType


class SLOListWidgetRequest(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.slo_list_widget_query import SLOListWidgetQuery
        from datadog_api_client.v1.model.slo_list_widget_request_type import SLOListWidgetRequestType

        return {
            "query": (SLOListWidgetQuery,),
            "request_type": (SLOListWidgetRequestType,),
        }

    attribute_map = {
        "query": "query",
        "request_type": "request_type",
    }

    def __init__(self_, query: SLOListWidgetQuery, request_type: SLOListWidgetRequestType, **kwargs):
        """
        Updated SLO List widget.

        :param query: Updated SLO List widget.
        :type query: SLOListWidgetQuery

        :param request_type: Widget request type.
        :type request_type: SLOListWidgetRequestType
        """
        super().__init__(kwargs)

        self_.query = query
        self_.request_type = request_type



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class WidgetEvent(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "q": (str,),
            "tags_execution": (str,),
        }

    attribute_map = {
        "q": "q",
        "tags_execution": "tags_execution",
    }

    def __init__(self_, q: str, tags_execution: Union[str, UnsetType] = unset, **kwargs):
        """
        Event overlay control options.

        See the dedicated `Events JSON schema documentation <https://docs.datadoghq.com/dashboards/graphing_json/widget_json/#events-schema>`_
        to learn how to build the ``<EVENTS_SCHEMA>``.

        :param q: Query definition.
        :type q: str

        :param tags_execution: The execution method for multi-value filters.
        :type tags_execution: str, optional
        """
        if tags_execution is not unset:
            kwargs["tags_execution"] = tags_execution
        super().__init__(kwargs)

        self_.q = q



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.sensitive_data_scanner_get_config_response_data import (
        SensitiveDataScannerGetConfigResponseData,
    )
    from datadog_api_client.v2.model.sensitive_data_scanner_get_config_included_array import (
        SensitiveDataScannerGetConfigIncludedArray,
    )
    from datadog_api_client.v2.model.sensitive_data_scanner_meta import SensitiveDataScannerMeta


class SensitiveDataScannerGetConfigResponse(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.sensitive_data_scanner_get_config_response_data import (
            SensitiveDataScannerGetConfigResponseData,
        )
        from datadog_api_client.v2.model.sensitive_data_scanner_get_config_included_array import (
            SensitiveDataScannerGetConfigIncludedArray,
        )
        from datadog_api_client.v2.model.sensitive_data_scanner_meta import SensitiveDataScannerMeta

        return {
            "data": (SensitiveDataScannerGetConfigResponseData,),
            "included": (SensitiveDataScannerGetConfigIncludedArray,),
            "meta": (SensitiveDataScannerMeta,),
        }

    attribute_map = {
        "data": "data",
        "included": "included",
        "meta": "meta",
    }

    def __init__(
        self_,
        data: Union[SensitiveDataScannerGetConfigResponseData, UnsetType] = unset,
        included: Union[SensitiveDataScannerGetConfigIncludedArray, UnsetType] = unset,
        meta: Union[SensitiveDataScannerMeta, UnsetType] = unset,
        **kwargs,
    ):
        """
        Get all groups response.

        :param data: Response data related to the scanning groups.
        :type data: SensitiveDataScannerGetConfigResponseData, optional

        :param included: Included objects from relationships.
        :type included: SensitiveDataScannerGetConfigIncludedArray, optional

        :param meta: Meta response containing information about the API.
        :type meta: SensitiveDataScannerMeta, optional
        """
        if data is not unset:
            kwargs["data"] = data
        if included is not unset:
            kwargs["included"] = included
        if meta is not unset:
            kwargs["meta"] = meta
        super().__init__(kwargs)



"""
Get specified daily custom reports returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.usage_metering_api import UsageMeteringApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = UsageMeteringApi(api_client)
    response = api_instance.get_specified_daily_custom_reports(
        report_id="2022-03-20",
    )

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


class NotebookSplitBy(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "keys": ([str],),
            "tags": ([str],),
        }

    attribute_map = {
        "keys": "keys",
        "tags": "tags",
    }

    def __init__(self_, keys: List[str], tags: List[str], **kwargs):
        """
        Object describing how to split the graph to display multiple visualizations per request.

        :param keys: Keys to split on.
        :type keys: [str]

        :param tags: Tags to split on.
        :type tags: [str]
        """
        super().__init__(kwargs)

        self_.keys = keys
        self_.tags = tags



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class WidgetTextAlign(ModelSimple):
    """
    How to align the text on the widget.

    :param value: Must be one of ["center", "left", "right"].
    :type value: str
    """

    allowed_values = {
        "center",
        "left",
        "right",
    }
    CENTER: ClassVar["WidgetTextAlign"]
    LEFT: ClassVar["WidgetTextAlign"]
    RIGHT: ClassVar["WidgetTextAlign"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


WidgetTextAlign.CENTER = WidgetTextAlign("center")
WidgetTextAlign.LEFT = WidgetTextAlign("left")
WidgetTextAlign.RIGHT = WidgetTextAlign("right")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class WidgetLineType(ModelSimple):
    """
    Type of lines displayed.

    :param value: Must be one of ["dashed", "dotted", "solid"].
    :type value: str
    """

    allowed_values = {
        "dashed",
        "dotted",
        "solid",
    }
    DASHED: ClassVar["WidgetLineType"]
    DOTTED: ClassVar["WidgetLineType"]
    SOLID: ClassVar["WidgetLineType"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


WidgetLineType.DASHED = WidgetLineType("dashed")
WidgetLineType.DOTTED = WidgetLineType("dotted")
WidgetLineType.SOLID = WidgetLineType("solid")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    datetime,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.event_attributes import EventAttributes


class EventResponseAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.event_attributes import EventAttributes

        return {
            "attributes": (EventAttributes,),
            "tags": ([str],),
            "timestamp": (datetime,),
        }

    attribute_map = {
        "attributes": "attributes",
        "tags": "tags",
        "timestamp": "timestamp",
    }

    def __init__(
        self_,
        attributes: Union[EventAttributes, UnsetType] = unset,
        tags: Union[List[str], UnsetType] = unset,
        timestamp: Union[datetime, UnsetType] = unset,
        **kwargs,
    ):
        """
        The object description of an event response attribute.

        :param attributes: Object description of attributes from your event.
        :type attributes: EventAttributes, optional

        :param tags: An array of tags associated with the event.
        :type tags: [str], optional

        :param timestamp: The timestamp of the event.
        :type timestamp: datetime, optional
        """
        if attributes is not unset:
            kwargs["attributes"] = attributes
        if tags is not unset:
            kwargs["tags"] = tags
        if timestamp is not unset:
            kwargs["timestamp"] = timestamp
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class AWSAccountDeleteRequest(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "access_key_id": (str,),
            "account_id": (str,),
            "role_name": (str,),
        }

    attribute_map = {
        "access_key_id": "access_key_id",
        "account_id": "account_id",
        "role_name": "role_name",
    }

    def __init__(
        self_,
        access_key_id: Union[str, UnsetType] = unset,
        account_id: Union[str, UnsetType] = unset,
        role_name: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        List of AWS accounts to delete.

        :param access_key_id: Your AWS access key ID. Only required if your AWS account is a GovCloud or China account.
        :type access_key_id: str, optional

        :param account_id: Your AWS Account ID without dashes.
        :type account_id: str, optional

        :param role_name: Your Datadog role delegation name.
        :type role_name: str, optional
        """
        if access_key_id is not unset:
            kwargs["access_key_id"] = access_key_id
        if account_id is not unset:
            kwargs["account_id"] = account_id
        if role_name is not unset:
            kwargs["role_name"] = role_name
        super().__init__(kwargs)



"""
Create a new timeseries widget with ci_tests data source
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.dashboard import Dashboard
from datadog_api_client.v1.model.dashboard_layout_type import DashboardLayoutType
from datadog_api_client.v1.model.dashboard_reflow_type import DashboardReflowType
from datadog_api_client.v1.model.formula_and_function_event_aggregation import FormulaAndFunctionEventAggregation
from datadog_api_client.v1.model.formula_and_function_event_query_definition import (
    FormulaAndFunctionEventQueryDefinition,
)
from datadog_api_client.v1.model.formula_and_function_event_query_definition_compute import (
    FormulaAndFunctionEventQueryDefinitionCompute,
)
from datadog_api_client.v1.model.formula_and_function_event_query_definition_search import (
    FormulaAndFunctionEventQueryDefinitionSearch,
)
from datadog_api_client.v1.model.formula_and_function_events_data_source import FormulaAndFunctionEventsDataSource
from datadog_api_client.v1.model.formula_and_function_response_format import FormulaAndFunctionResponseFormat
from datadog_api_client.v1.model.timeseries_widget_definition import TimeseriesWidgetDefinition
from datadog_api_client.v1.model.timeseries_widget_definition_type import TimeseriesWidgetDefinitionType
from datadog_api_client.v1.model.timeseries_widget_legend_column import TimeseriesWidgetLegendColumn
from datadog_api_client.v1.model.timeseries_widget_legend_layout import TimeseriesWidgetLegendLayout
from datadog_api_client.v1.model.timeseries_widget_request import TimeseriesWidgetRequest
from datadog_api_client.v1.model.widget import Widget
from datadog_api_client.v1.model.widget_display_type import WidgetDisplayType
from datadog_api_client.v1.model.widget_formula import WidgetFormula
from datadog_api_client.v1.model.widget_line_type import WidgetLineType
from datadog_api_client.v1.model.widget_line_width import WidgetLineWidth
from datadog_api_client.v1.model.widget_request_style import WidgetRequestStyle
from datadog_api_client.v1.model.widget_time import WidgetTime

body = Dashboard(
    title="Example-Create_a_new_timeseries_widget_with_ci_tests_data_source with ci_tests datasource",
    widgets=[
        Widget(
            definition=TimeseriesWidgetDefinition(
                title="",
                show_legend=True,
                legend_layout=TimeseriesWidgetLegendLayout.AUTO,
                legend_columns=[
                    TimeseriesWidgetLegendColumn.AVG,
                    TimeseriesWidgetLegendColumn.MIN,
                    TimeseriesWidgetLegendColumn.MAX,
                    TimeseriesWidgetLegendColumn.VALUE,
                    TimeseriesWidgetLegendColumn.SUM,
                ],
                time=WidgetTime(),
                type=TimeseriesWidgetDefinitionType.TIMESERIES,
                requests=[
                    TimeseriesWidgetRequest(
                        formulas=[
                            WidgetFormula(
                                formula="query1",
                            ),
                        ],
                        queries=[
                            FormulaAndFunctionEventQueryDefinition(
                                data_source=FormulaAndFunctionEventsDataSource.CI_TESTS,
                                name="query1",
                                search=FormulaAndFunctionEventQueryDefinitionSearch(
                                    query="test_level:test",
                                ),
                                indexes=[
                                    "*",
                                ],
                                compute=FormulaAndFunctionEventQueryDefinitionCompute(
                                    aggregation=FormulaAndFunctionEventAggregation.COUNT,
                                ),
                                group_by=[],
                            ),
                        ],
                        response_format=FormulaAndFunctionResponseFormat.TIMESERIES,
                        style=WidgetRequestStyle(
                            palette="dog_classic",
                            line_type=WidgetLineType.SOLID,
                            line_width=WidgetLineWidth.NORMAL,
                        ),
                        display_type=WidgetDisplayType.LINE,
                    ),
                ],
            ),
        ),
    ],
    layout_type=DashboardLayoutType.ORDERED,
    reflow_type=DashboardReflowType.AUTO,
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    response = api_instance.create_dashboard(body=body)

    print(response)



"""
Get a list of incidents returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.incidents_api import IncidentsApi

configuration = Configuration()
configuration.unstable_operations["list_incidents"] = True
with ApiClient(configuration) as api_client:
    api_instance = IncidentsApi(api_client)
    response = api_instance.list_incidents()

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.sensitive_data_scanner_text_replacement import SensitiveDataScannerTextReplacement


class SensitiveDataScannerRuleAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.sensitive_data_scanner_text_replacement import (
            SensitiveDataScannerTextReplacement,
        )

        return {
            "description": (str,),
            "excluded_attributes": ([str],),
            "is_enabled": (bool,),
            "name": (str,),
            "pattern": (str,),
            "tags": ([str],),
            "text_replacement": (SensitiveDataScannerTextReplacement,),
        }

    attribute_map = {
        "description": "description",
        "excluded_attributes": "excluded_attributes",
        "is_enabled": "is_enabled",
        "name": "name",
        "pattern": "pattern",
        "tags": "tags",
        "text_replacement": "text_replacement",
    }

    def __init__(
        self_,
        description: Union[str, UnsetType] = unset,
        excluded_attributes: Union[List[str], UnsetType] = unset,
        is_enabled: Union[bool, UnsetType] = unset,
        name: Union[str, UnsetType] = unset,
        pattern: Union[str, UnsetType] = unset,
        tags: Union[List[str], UnsetType] = unset,
        text_replacement: Union[SensitiveDataScannerTextReplacement, UnsetType] = unset,
        **kwargs,
    ):
        """
        Attributes of the Sensitive Data Scanner rule.

        :param description: Description of the rule.
        :type description: str, optional

        :param excluded_attributes: Attributes excluded from the scan.
        :type excluded_attributes: [str], optional

        :param is_enabled: Whether or not the rule is enabled.
        :type is_enabled: bool, optional

        :param name: Name of the rule.
        :type name: str, optional

        :param pattern: Not included if there is a relationship to a standard pattern.
        :type pattern: str, optional

        :param tags: List of tags.
        :type tags: [str], optional

        :param text_replacement: Object describing how the scanned event will be replaced.
        :type text_replacement: SensitiveDataScannerTextReplacement, optional
        """
        if description is not unset:
            kwargs["description"] = description
        if excluded_attributes is not unset:
            kwargs["excluded_attributes"] = excluded_attributes
        if is_enabled is not unset:
            kwargs["is_enabled"] = is_enabled
        if name is not unset:
            kwargs["name"] = name
        if pattern is not unset:
            kwargs["pattern"] = pattern
        if tags is not unset:
            kwargs["tags"] = tags
        if text_replacement is not unset:
            kwargs["text_replacement"] = text_replacement
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.synthetics_parsing_options import SyntheticsParsingOptions
    from datadog_api_client.v1.model.synthetics_test_request import SyntheticsTestRequest
    from datadog_api_client.v1.model.synthetics_test_options_retry import SyntheticsTestOptionsRetry
    from datadog_api_client.v1.model.synthetics_api_step_subtype import SyntheticsAPIStepSubtype


class SyntheticsAPIStep(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.synthetics_assertion import SyntheticsAssertion
        from datadog_api_client.v1.model.synthetics_parsing_options import SyntheticsParsingOptions
        from datadog_api_client.v1.model.synthetics_test_request import SyntheticsTestRequest
        from datadog_api_client.v1.model.synthetics_test_options_retry import SyntheticsTestOptionsRetry
        from datadog_api_client.v1.model.synthetics_api_step_subtype import SyntheticsAPIStepSubtype

        return {
            "allow_failure": (bool,),
            "assertions": ([SyntheticsAssertion],),
            "extracted_values": ([SyntheticsParsingOptions],),
            "is_critical": (bool,),
            "name": (str,),
            "request": (SyntheticsTestRequest,),
            "retry": (SyntheticsTestOptionsRetry,),
            "subtype": (SyntheticsAPIStepSubtype,),
        }

    attribute_map = {
        "allow_failure": "allowFailure",
        "assertions": "assertions",
        "extracted_values": "extractedValues",
        "is_critical": "isCritical",
        "name": "name",
        "request": "request",
        "retry": "retry",
        "subtype": "subtype",
    }

    def __init__(
        self_,
        name: str,
        request: SyntheticsTestRequest,
        subtype: SyntheticsAPIStepSubtype,
        allow_failure: Union[bool, UnsetType] = unset,
        extracted_values: Union[List[SyntheticsParsingOptions], UnsetType] = unset,
        is_critical: Union[bool, UnsetType] = unset,
        retry: Union[SyntheticsTestOptionsRetry, UnsetType] = unset,
        **kwargs,
    ):
        """
        The steps used in a Synthetics multistep API test.

        :param allow_failure: Determines whether or not to continue with test if this step fails.
        :type allow_failure: bool, optional

        :param assertions: Array of assertions used for the test.
        :type assertions: [SyntheticsAssertion]

        :param extracted_values: Array of values to parse and save as variables from the response.
        :type extracted_values: [SyntheticsParsingOptions], optional

        :param is_critical: Determines whether or not to consider the entire test as failed if this step fails.
            Can be used only if ``allowFailure`` is ``true``.
        :type is_critical: bool, optional

        :param name: The name of the step.
        :type name: str

        :param request: Object describing the Synthetic test request.
        :type request: SyntheticsTestRequest

        :param retry: Object describing the retry strategy to apply to a Synthetic test.
        :type retry: SyntheticsTestOptionsRetry, optional

        :param subtype: The subtype of the Synthetic multistep API test step, currently only supporting ``http``.
        :type subtype: SyntheticsAPIStepSubtype
        """
        if allow_failure is not unset:
            kwargs["allow_failure"] = allow_failure
        if extracted_values is not unset:
            kwargs["extracted_values"] = extracted_values
        if is_critical is not unset:
            kwargs["is_critical"] = is_critical
        if retry is not unset:
            kwargs["retry"] = retry
        super().__init__(kwargs)
        assertions = kwargs.get("assertions", [])

        self_.assertions = assertions
        self_.name = name
        self_.request = request
        self_.subtype = subtype



"""
Get API key returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.key_management_api import KeyManagementApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = KeyManagementApi(api_client)
    response = api_instance.get_api_key(
        key="key",
    )

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class MetricDistinctVolumeAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "distinct_volume": (int,),
        }

    attribute_map = {
        "distinct_volume": "distinct_volume",
    }

    def __init__(self_, distinct_volume: Union[int, UnsetType] = unset, **kwargs):
        """
        Object containing the definition of a metric's distinct volume.

        :param distinct_volume: Distinct volume for the given metric.
        :type distinct_volume: int, optional
        """
        if distinct_volume is not unset:
            kwargs["distinct_volume"] = distinct_volume
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class MonitorOptionsAggregation(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "group_by": (str,),
            "metric": (str,),
            "type": (str,),
        }

    attribute_map = {
        "group_by": "group_by",
        "metric": "metric",
        "type": "type",
    }

    def __init__(
        self_,
        group_by: Union[str, UnsetType] = unset,
        metric: Union[str, UnsetType] = unset,
        type: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        Type of aggregation performed in the monitor query.

        :param group_by: Group to break down the monitor on.
        :type group_by: str, optional

        :param metric: Metric name used in the monitor.
        :type metric: str, optional

        :param type: Metric type used in the monitor.
        :type type: str, optional
        """
        if group_by is not unset:
            kwargs["group_by"] = group_by
        if metric is not unset:
            kwargs["metric"] = metric
        if type is not unset:
            kwargs["type"] = type
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class OpsgenieServiceRegionType(ModelSimple):
    """
    The region for the Opsgenie service.

    :param value: Must be one of ["us", "eu", "custom"].
    :type value: str
    """

    allowed_values = {
        "us",
        "eu",
        "custom",
    }
    US: ClassVar["OpsgenieServiceRegionType"]
    EU: ClassVar["OpsgenieServiceRegionType"]
    CUSTOM: ClassVar["OpsgenieServiceRegionType"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


OpsgenieServiceRegionType.US = OpsgenieServiceRegionType("us")
OpsgenieServiceRegionType.EU = OpsgenieServiceRegionType("eu")
OpsgenieServiceRegionType.CUSTOM = OpsgenieServiceRegionType("custom")



"""
Search logs returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.logs_api import LogsApi
from datadog_api_client.v2.model.logs_list_request import LogsListRequest
from datadog_api_client.v2.model.logs_list_request_page import LogsListRequestPage
from datadog_api_client.v2.model.logs_query_filter import LogsQueryFilter
from datadog_api_client.v2.model.logs_sort import LogsSort

body = LogsListRequest(
    filter=LogsQueryFilter(
        query="datadog-agent",
        indexes=[
            "main",
        ],
        _from="2020-09-17T11:48:36+01:00",
        to="2020-09-17T12:48:36+01:00",
    ),
    sort=LogsSort.TIMESTAMP_ASCENDING,
    page=LogsListRequestPage(
        limit=5,
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = LogsApi(api_client)
    response = api_instance.list_logs(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    datetime,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.creator import Creator
    from datadog_api_client.v2.model.dashboard_type import DashboardType


class DashboardListItem(ModelNormal):
    validations = {
        "popularity": {
            "inclusive_maximum": 5,
        },
    }

    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.creator import Creator
        from datadog_api_client.v2.model.dashboard_type import DashboardType

        return {
            "author": (Creator,),
            "created": (datetime,),
            "icon": (str,),
            "id": (str,),
            "is_favorite": (bool,),
            "is_read_only": (bool,),
            "is_shared": (bool,),
            "modified": (datetime,),
            "popularity": (int,),
            "title": (str,),
            "type": (DashboardType,),
            "url": (str,),
        }

    attribute_map = {
        "author": "author",
        "created": "created",
        "icon": "icon",
        "id": "id",
        "is_favorite": "is_favorite",
        "is_read_only": "is_read_only",
        "is_shared": "is_shared",
        "modified": "modified",
        "popularity": "popularity",
        "title": "title",
        "type": "type",
        "url": "url",
    }
    read_only_vars = {
        "created",
        "icon",
        "is_favorite",
        "is_read_only",
        "is_shared",
        "modified",
        "popularity",
        "title",
        "url",
    }

    def __init__(
        self_,
        id: str,
        type: DashboardType,
        author: Union[Creator, UnsetType] = unset,
        created: Union[datetime, UnsetType] = unset,
        icon: Union[str, UnsetType] = unset,
        is_favorite: Union[bool, UnsetType] = unset,
        is_read_only: Union[bool, UnsetType] = unset,
        is_shared: Union[bool, UnsetType] = unset,
        modified: Union[datetime, UnsetType] = unset,
        popularity: Union[int, UnsetType] = unset,
        title: Union[str, UnsetType] = unset,
        url: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        A dashboard within a list.

        :param author: Creator of the object.
        :type author: Creator, optional

        :param created: Date of creation of the dashboard.
        :type created: datetime, optional

        :param icon: URL to the icon of the dashboard.
        :type icon: str, optional

        :param id: ID of the dashboard.
        :type id: str

        :param is_favorite: Whether or not the dashboard is in the favorites.
        :type is_favorite: bool, optional

        :param is_read_only: Whether or not the dashboard is read only.
        :type is_read_only: bool, optional

        :param is_shared: Whether the dashboard is publicly shared or not.
        :type is_shared: bool, optional

        :param modified: Date of last edition of the dashboard.
        :type modified: datetime, optional

        :param popularity: Popularity of the dashboard.
        :type popularity: int, optional

        :param title: Title of the dashboard.
        :type title: str, optional

        :param type: The type of the dashboard.
        :type type: DashboardType

        :param url: URL path to the dashboard.
        :type url: str, optional
        """
        if author is not unset:
            kwargs["author"] = author
        if created is not unset:
            kwargs["created"] = created
        if icon is not unset:
            kwargs["icon"] = icon
        if is_favorite is not unset:
            kwargs["is_favorite"] = is_favorite
        if is_read_only is not unset:
            kwargs["is_read_only"] = is_read_only
        if is_shared is not unset:
            kwargs["is_shared"] = is_shared
        if modified is not unset:
            kwargs["modified"] = modified
        if popularity is not unset:
            kwargs["popularity"] = popularity
        if title is not unset:
            kwargs["title"] = title
        if url is not unset:
            kwargs["url"] = url
        super().__init__(kwargs)

        self_.id = id
        self_.type = type



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.usage_attribution_aggregates import UsageAttributionAggregates
    from datadog_api_client.v1.model.usage_attribution_pagination import UsageAttributionPagination


class UsageAttributionMetadata(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.usage_attribution_aggregates import UsageAttributionAggregates
        from datadog_api_client.v1.model.usage_attribution_pagination import UsageAttributionPagination

        return {
            "aggregates": (UsageAttributionAggregates,),
            "pagination": (UsageAttributionPagination,),
        }

    attribute_map = {
        "aggregates": "aggregates",
        "pagination": "pagination",
    }

    def __init__(
        self_,
        aggregates: Union[UsageAttributionAggregates, UnsetType] = unset,
        pagination: Union[UsageAttributionPagination, UnsetType] = unset,
        **kwargs,
    ):
        """
        The object containing document metadata.

        :param aggregates: An array of available aggregates.
        :type aggregates: UsageAttributionAggregates, optional

        :param pagination: The metadata for the current pagination.
        :type pagination: UsageAttributionPagination, optional
        """
        if aggregates is not unset:
            kwargs["aggregates"] = aggregates
        if pagination is not unset:
            kwargs["pagination"] = pagination
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class WidgetStyle(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "palette": (str,),
        }

    attribute_map = {
        "palette": "palette",
    }

    def __init__(self_, palette: Union[str, UnsetType] = unset, **kwargs):
        """
        Widget style definition.

        :param palette: Color palette to apply to the widget.
        :type palette: str, optional
        """
        if palette is not unset:
            kwargs["palette"] = palette
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelComposed,
    cached_property,
)


class SecurityMonitoringRuleResponse(ModelComposed):
    def __init__(self, **kwargs):
        """
        Create a new rule.

        :param cases: Cases for generating signals.
        :type cases: [SecurityMonitoringRuleCase], optional

        :param compliance_signal_options: How to generate compliance signals. Useful for cloud_configuration rules only.
        :type compliance_signal_options: CloudConfigurationRuleComplianceSignalOptions, optional

        :param created_at: When the rule was created, timestamp in milliseconds.
        :type created_at: int, optional

        :param creation_author_id: User ID of the user who created the rule.
        :type creation_author_id: int, optional

        :param filters: Additional queries to filter matched events before they are processed.
        :type filters: [SecurityMonitoringFilter], optional

        :param has_extended_title: Whether the notifications include the triggering group-by values in their title.
        :type has_extended_title: bool, optional

        :param id: The ID of the rule.
        :type id: str, optional

        :param is_default: Whether the rule is included by default.
        :type is_default: bool, optional

        :param is_deleted: Whether the rule has been deleted.
        :type is_deleted: bool, optional

        :param is_enabled: Whether the rule is enabled.
        :type is_enabled: bool, optional

        :param message: Message for generated signals.
        :type message: str, optional

        :param name: The name of the rule.
        :type name: str, optional

        :param options: Options on rules.
        :type options: SecurityMonitoringRuleOptions, optional

        :param queries: Queries for selecting logs which are part of the rule.
        :type queries: [SecurityMonitoringStandardRuleQuery], optional

        :param tags: Tags for generated signals.
        :type tags: [str], optional

        :param type: The rule type.
        :type type: SecurityMonitoringRuleTypeRead, optional

        :param update_author_id: User ID of the user who updated the rule.
        :type update_author_id: int, optional

        :param version: The version of the rule.
        :type version: int, optional
        """
        super().__init__(kwargs)

    @cached_property
    def _composed_schemas(_):
        # we need this here to make our import statements work
        # we must store _composed_schemas in here so the code is only run
        # when we invoke this method. If we kept this at the class
        # level we would get an error because the class level
        # code would be run when this module is imported, and these composed
        # classes don't exist yet because their module has not finished
        # loading
        from datadog_api_client.v2.model.security_monitoring_standard_rule_response import (
            SecurityMonitoringStandardRuleResponse,
        )
        from datadog_api_client.v2.model.security_monitoring_signal_rule_response import (
            SecurityMonitoringSignalRuleResponse,
        )

        return {
            "oneOf": [
                SecurityMonitoringStandardRuleResponse,
                SecurityMonitoringSignalRuleResponse,
            ],
        }



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Any, Dict, Union

from datadog_api_client.api_client import ApiClient, Endpoint as _Endpoint
from datadog_api_client.model_utils import (
    UnsetType,
    unset,
)
from datadog_api_client.v2.model.permissions_response import PermissionsResponse
from datadog_api_client.v2.model.roles_response import RolesResponse
from datadog_api_client.v2.model.roles_sort import RolesSort
from datadog_api_client.v2.model.role_create_response import RoleCreateResponse
from datadog_api_client.v2.model.role_create_request import RoleCreateRequest
from datadog_api_client.v2.model.role_response import RoleResponse
from datadog_api_client.v2.model.role_update_response import RoleUpdateResponse
from datadog_api_client.v2.model.role_update_request import RoleUpdateRequest
from datadog_api_client.v2.model.role_clone_request import RoleCloneRequest
from datadog_api_client.v2.model.relationship_to_permission import RelationshipToPermission
from datadog_api_client.v2.model.users_response import UsersResponse
from datadog_api_client.v2.model.relationship_to_user import RelationshipToUser


class RolesApi:
    """
    The Roles API is used to create and manage Datadog roles, what
    `global permissions <https://docs.datadoghq.com/account_management/rbac/>`_
    they grant, and which users belong to them.

    Permissions related to specific account assets can be granted to roles
    in the Datadog application without using this API. For example, granting
    read access on a specific log index to a role can be done in Datadog from the
    `Pipelines page <https://app.datadoghq.com/logs/pipelines>`_.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self._add_permission_to_role_endpoint = _Endpoint(
            settings={
                "response_type": (PermissionsResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles/{role_id}/permissions",
                "operation_id": "add_permission_to_role",
                "http_method": "POST",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "role_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "role_id",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (RelationshipToPermission,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._add_user_to_role_endpoint = _Endpoint(
            settings={
                "response_type": (UsersResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles/{role_id}/users",
                "operation_id": "add_user_to_role",
                "http_method": "POST",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "role_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "role_id",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (RelationshipToUser,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._clone_role_endpoint = _Endpoint(
            settings={
                "response_type": (RoleResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles/{role_id}/clone",
                "operation_id": "clone_role",
                "http_method": "POST",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "role_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "role_id",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (RoleCloneRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._create_role_endpoint = _Endpoint(
            settings={
                "response_type": (RoleCreateResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles",
                "operation_id": "create_role",
                "http_method": "POST",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (RoleCreateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._delete_role_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles/{role_id}",
                "operation_id": "delete_role",
                "http_method": "DELETE",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "role_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "role_id",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["*/*"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._get_role_endpoint = _Endpoint(
            settings={
                "response_type": (RoleResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles/{role_id}",
                "operation_id": "get_role",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "role_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "role_id",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_permissions_endpoint = _Endpoint(
            settings={
                "response_type": (PermissionsResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/permissions",
                "operation_id": "list_permissions",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={},
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_role_permissions_endpoint = _Endpoint(
            settings={
                "response_type": (PermissionsResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles/{role_id}/permissions",
                "operation_id": "list_role_permissions",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "role_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "role_id",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_roles_endpoint = _Endpoint(
            settings={
                "response_type": (RolesResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles",
                "operation_id": "list_roles",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "page_size": {
                    "openapi_types": (int,),
                    "attribute": "page[size]",
                    "location": "query",
                },
                "page_number": {
                    "openapi_types": (int,),
                    "attribute": "page[number]",
                    "location": "query",
                },
                "sort": {
                    "openapi_types": (RolesSort,),
                    "attribute": "sort",
                    "location": "query",
                },
                "filter": {
                    "openapi_types": (str,),
                    "attribute": "filter",
                    "location": "query",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_role_users_endpoint = _Endpoint(
            settings={
                "response_type": (UsersResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles/{role_id}/users",
                "operation_id": "list_role_users",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "role_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "role_id",
                    "location": "path",
                },
                "page_size": {
                    "openapi_types": (int,),
                    "attribute": "page[size]",
                    "location": "query",
                },
                "page_number": {
                    "openapi_types": (int,),
                    "attribute": "page[number]",
                    "location": "query",
                },
                "sort": {
                    "openapi_types": (str,),
                    "attribute": "sort",
                    "location": "query",
                },
                "filter": {
                    "openapi_types": (str,),
                    "attribute": "filter",
                    "location": "query",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._remove_permission_from_role_endpoint = _Endpoint(
            settings={
                "response_type": (PermissionsResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles/{role_id}/permissions",
                "operation_id": "remove_permission_from_role",
                "http_method": "DELETE",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "role_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "role_id",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (RelationshipToPermission,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._remove_user_from_role_endpoint = _Endpoint(
            settings={
                "response_type": (UsersResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles/{role_id}/users",
                "operation_id": "remove_user_from_role",
                "http_method": "DELETE",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "role_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "role_id",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (RelationshipToUser,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._update_role_endpoint = _Endpoint(
            settings={
                "response_type": (RoleUpdateResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/roles/{role_id}",
                "operation_id": "update_role",
                "http_method": "PATCH",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "role_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "role_id",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (RoleUpdateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

    def add_permission_to_role(
        self,
        role_id: str,
        body: RelationshipToPermission,
    ) -> PermissionsResponse:
        """Grant permission to a role.

        Adds a permission to a role.

        :param role_id: The unique identifier of the role.
        :type role_id: str
        :type body: RelationshipToPermission
        :rtype: PermissionsResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["role_id"] = role_id

        kwargs["body"] = body

        return self._add_permission_to_role_endpoint.call_with_http_info(**kwargs)

    def add_user_to_role(
        self,
        role_id: str,
        body: RelationshipToUser,
    ) -> UsersResponse:
        """Add a user to a role.

        Adds a user to a role.

        :param role_id: The unique identifier of the role.
        :type role_id: str
        :type body: RelationshipToUser
        :rtype: UsersResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["role_id"] = role_id

        kwargs["body"] = body

        return self._add_user_to_role_endpoint.call_with_http_info(**kwargs)

    def clone_role(
        self,
        role_id: str,
        body: RoleCloneRequest,
    ) -> RoleResponse:
        """Create a new role by cloning an existing role.

        Clone an existing role

        :param role_id: The unique identifier of the role.
        :type role_id: str
        :type body: RoleCloneRequest
        :rtype: RoleResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["role_id"] = role_id

        kwargs["body"] = body

        return self._clone_role_endpoint.call_with_http_info(**kwargs)

    def create_role(
        self,
        body: RoleCreateRequest,
    ) -> RoleCreateResponse:
        """Create role.

        Create a new role for your organization.

        :type body: RoleCreateRequest
        :rtype: RoleCreateResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._create_role_endpoint.call_with_http_info(**kwargs)

    def delete_role(
        self,
        role_id: str,
    ) -> None:
        """Delete role.

        Disables a role.

        :param role_id: The unique identifier of the role.
        :type role_id: str
        :rtype: None
        """
        kwargs: Dict[str, Any] = {}
        kwargs["role_id"] = role_id

        return self._delete_role_endpoint.call_with_http_info(**kwargs)

    def get_role(
        self,
        role_id: str,
    ) -> RoleResponse:
        """Get a role.

        Get a role in the organization specified by the role’s ``role_id``.

        :param role_id: The unique identifier of the role.
        :type role_id: str
        :rtype: RoleResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["role_id"] = role_id

        return self._get_role_endpoint.call_with_http_info(**kwargs)

    def list_permissions(
        self,
    ) -> PermissionsResponse:
        """List permissions.

        Returns a list of all permissions, including name, description, and ID.

        :rtype: PermissionsResponse
        """
        kwargs: Dict[str, Any] = {}
        return self._list_permissions_endpoint.call_with_http_info(**kwargs)

    def list_role_permissions(
        self,
        role_id: str,
    ) -> PermissionsResponse:
        """List permissions for a role.

        Returns a list of all permissions for a single role.

        :param role_id: The unique identifier of the role.
        :type role_id: str
        :rtype: PermissionsResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["role_id"] = role_id

        return self._list_role_permissions_endpoint.call_with_http_info(**kwargs)

    def list_roles(
        self,
        *,
        page_size: Union[int, UnsetType] = unset,
        page_number: Union[int, UnsetType] = unset,
        sort: Union[RolesSort, UnsetType] = unset,
        filter: Union[str, UnsetType] = unset,
    ) -> RolesResponse:
        """List roles.

        Returns all roles, including their names and their unique identifiers.

        :param page_size: Size for a given page. The maximum allowed value is 5000.
        :type page_size: int, optional
        :param page_number: Specific page number to return.
        :type page_number: int, optional
        :param sort: Sort roles depending on the given field. Sort order is **ascending** by default.
            Sort order is **descending** if the field is prefixed by a negative sign, for example:
            ``sort=-name``.
        :type sort: RolesSort, optional
        :param filter: Filter all roles by the given string.
        :type filter: str, optional
        :rtype: RolesResponse
        """
        kwargs: Dict[str, Any] = {}
        if page_size is not unset:
            kwargs["page_size"] = page_size

        if page_number is not unset:
            kwargs["page_number"] = page_number

        if sort is not unset:
            kwargs["sort"] = sort

        if filter is not unset:
            kwargs["filter"] = filter

        return self._list_roles_endpoint.call_with_http_info(**kwargs)

    def list_role_users(
        self,
        role_id: str,
        *,
        page_size: Union[int, UnsetType] = unset,
        page_number: Union[int, UnsetType] = unset,
        sort: Union[str, UnsetType] = unset,
        filter: Union[str, UnsetType] = unset,
    ) -> UsersResponse:
        """Get all users of a role.

        Gets all users of a role.

        :param role_id: The unique identifier of the role.
        :type role_id: str
        :param page_size: Size for a given page. The maximum allowed value is 5000.
        :type page_size: int, optional
        :param page_number: Specific page number to return.
        :type page_number: int, optional
        :param sort: User attribute to order results by. Sort order is **ascending** by default.
            Sort order is **descending** if the field is prefixed by a negative sign,
            for example ``sort=-name``. Options: ``name`` , ``email`` , ``status``.
        :type sort: str, optional
        :param filter: Filter all users by the given string. Defaults to no filtering.
        :type filter: str, optional
        :rtype: UsersResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["role_id"] = role_id

        if page_size is not unset:
            kwargs["page_size"] = page_size

        if page_number is not unset:
            kwargs["page_number"] = page_number

        if sort is not unset:
            kwargs["sort"] = sort

        if filter is not unset:
            kwargs["filter"] = filter

        return self._list_role_users_endpoint.call_with_http_info(**kwargs)

    def remove_permission_from_role(
        self,
        role_id: str,
        body: RelationshipToPermission,
    ) -> PermissionsResponse:
        """Revoke permission.

        Removes a permission from a role.

        :param role_id: The unique identifier of the role.
        :type role_id: str
        :type body: RelationshipToPermission
        :rtype: PermissionsResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["role_id"] = role_id

        kwargs["body"] = body

        return self._remove_permission_from_role_endpoint.call_with_http_info(**kwargs)

    def remove_user_from_role(
        self,
        role_id: str,
        body: RelationshipToUser,
    ) -> UsersResponse:
        """Remove a user from a role.

        Removes a user from a role.

        :param role_id: The unique identifier of the role.
        :type role_id: str
        :type body: RelationshipToUser
        :rtype: UsersResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["role_id"] = role_id

        kwargs["body"] = body

        return self._remove_user_from_role_endpoint.call_with_http_info(**kwargs)

    def update_role(
        self,
        role_id: str,
        body: RoleUpdateRequest,
    ) -> RoleUpdateResponse:
        """Update a role.

        Edit a role. Can only be used with application keys belonging to administrators.

        :param role_id: The unique identifier of the role.
        :type role_id: str
        :type body: RoleUpdateRequest
        :rtype: RoleUpdateResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["role_id"] = role_id

        kwargs["body"] = body

        return self._update_role_endpoint.call_with_http_info(**kwargs)



# coding=utf-8
"""Define basic fixtures."""

import os

RECORD = os.getenv("RECORD", "false").lower()
SLEEP_AFTER_REQUEST = int(os.getenv("SLEEP_AFTER_REQUEST", "0"))

# First patch urllib
tracer = None
try:
    from ddtrace import patch, tracer

    if RECORD != "none":
        from ddtrace.internal.writer import AgentWriter

        writer = AgentWriter(
            tracer._writer.agent_url,
            sync_mode=True,
            priority_sampler=tracer._priority_sampler,
        )
        tracer.configure(writer)

    patch(urllib3=True)

    from pytest import hookimpl

    @hookimpl(hookwrapper=True)
    def pytest_terminal_summary(terminalreporter, exitstatus, config):
        yield  # do normal output

        ci_pipeline_id = os.getenv("GITHUB_RUN_ID", None)
        dd_service = os.getenv("DD_SERVICE", None)
        if ci_pipeline_id and dd_service:
            terminalreporter.ensure_newline()
            terminalreporter.section("test reports", purple=True, bold=True)
            terminalreporter.line(
                "* View test APM traces and detailed time reports on Datadog (can take a few minutes to become available):"
            )
            terminalreporter.line(
                "* https://app.datadoghq.com/ci/test-runs?query="
                "%40test.service%3A{}%20%40ci.pipeline.id%3A{}&index=citest".format(dd_service, ci_pipeline_id)
            )

except ImportError:
    if os.getenv("CI", "false") == "true" and RECORD == "none":
        raise

import importlib
import functools
import json
import logging
import pathlib
import re
import time
import warnings
from datetime import datetime

import pytest
from dateutil.relativedelta import relativedelta
from jinja2 import Template, Environment, meta
from pytest_bdd import given, parsers, then, when

from datadog_api_client import exceptions
from datadog_api_client.api_client import ApiClient
from datadog_api_client.configuration import Configuration

logging.basicConfig()

with (pathlib.Path(__file__).parent.parent / ".generator" / "src" / "generator" / "replacement.json").open() as f:
    EDGE_CASES = json.load(f)

PATTERN_ALPHANUM = re.compile(r"[^A-Za-z0-9]+")
PATTERN_DOUBLE_UNDERSCORE = re.compile(r"__+")
PATTERN_LEADING_ALPHA = re.compile(r"(.)([A-Z][a-z]+)")
PATTERN_FOLLOWING_ALPHA = re.compile(r"([a-z0-9])([A-Z])")
PATTERN_WHITESPACE = re.compile(r"\W")
PATTERN_INDEX = re.compile(r"\[([0-9]*)\]")


def sleep_after_request(f):
    """Sleep after each request."""
    if RECORD == "false" or SLEEP_AFTER_REQUEST <= 0:
        return f

    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        result = f(*args, **kwargs)
        time.sleep(SLEEP_AFTER_REQUEST)
        return result

    return wrapper


def escape_reserved_keyword(word):
    """Escape reserved language keywords like openapi generator does it.

    :param word: Word to escape
    :return: The escaped word if it was a reserved keyword, the word unchanged otherwise
    """
    reserved_keywords = ["from"]
    if word in reserved_keywords:
        return f"_{word}"
    return word


def pytest_bdd_after_scenario(request, feature, scenario):
    try:
        ctx = request.getfixturevalue("context")
    except Exception:
        return
    for undo in reversed(ctx["undo_operations"]):
        undo()


def pytest_bdd_apply_tag(tag, function):
    """Register tags as custom markers and skip test for '@skip' ones."""
    skip_tags = {"skip", "skip-python"}
    if RECORD != "none":
        # ignore integration-only scenarios if the recording is enabled
        skip_tags.add("integration-only")
    if RECORD != "false":
        skip_tags.add("replay-only")

    if tag in skip_tags:
        marker = pytest.mark.skip(reason=f"skipped because of '{tag} in {skip_tags}")
        marker(function)
    return True


def snake_case(value):
    for token, replacement in EDGE_CASES.items():
        value = value.replace(token, replacement)
    s1 = PATTERN_LEADING_ALPHA.sub(r"\1_\2", value)
    s1 = PATTERN_FOLLOWING_ALPHA.sub(r"\1_\2", s1).lower()
    s1 = PATTERN_WHITESPACE.sub("_", s1)
    s1 = s1.rstrip("_")
    return PATTERN_DOUBLE_UNDERSCORE.sub("_", s1)


def glom(value, path):
    from glom import glom as g

    # replace foo[index].bar by foo.index.bar
    path = PATTERN_INDEX.sub(r".\1", path)
    if not isinstance(value, dict):
        path = ".".join(snake_case(p) for p in path.split("."))

    return g(value, path) if path else value


def _get_prefix(request):
    test_class = request.cls
    if test_class:
        main = "{}.{}".format(test_class.__name__, request.node.name)
    else:
        base_name = request.node.__scenario_report__.scenario.name
        main = PATTERN_ALPHANUM.sub("_", base_name)[:100]
    prefix = "Test-Python" if _disable_recording() else "Test"
    return f"{prefix}-{main}"


@pytest.fixture
def unique(request, freezed_time):
    prefix = _get_prefix(request)
    return f"{prefix}-{int(freezed_time.timestamp())}"


def relative_time(freezed_time, iso):
    time_re = re.compile(r"now( *([+-]) *(\d+)([smhdMy]))?")

    def func(arg):
        ret = freezed_time
        m = time_re.match(arg)
        if m:
            if m.group(1):
                sign = m.group(2)
                num = int(sign + m.group(3))
                unit = m.group(4)
                if unit == "s":
                    ret += relativedelta(seconds=num)
                elif unit == "m":
                    ret += relativedelta(minutes=num)
                elif unit == "h":
                    ret += relativedelta(hours=num)
                elif unit == "d":
                    ret += relativedelta(days=num)
                elif unit == "M":
                    ret += relativedelta(months=num)
                elif unit == "y":
                    ret += relativedelta(years=num)
            if iso:
                return ret.replace(tzinfo=None)  # return datetime object and not string
                # NOTE this is not a full ISO 8601 format, but it's enough for our needs
                # return ret.strftime('%Y-%m-%dT%H:%M:%S') + ret.strftime('.%f')[:4] + 'Z'
            return int(ret.timestamp())
        return ""

    return func


@pytest.fixture
def context(vcr, unique, freezed_time):
    """
    Return a mapping with all defined fixtures, all objects created by `given` steps,
    and the undo operations to perform after a test scenario.
    """
    ctx = {
        "undo_operations": [],
        "unique": unique,
        "unique_lower": unique.lower(),
        "unique_upper": unique.upper(),
        "unique_alnum": PATTERN_ALPHANUM.sub("", unique),
        "unique_lower_alnum": PATTERN_ALPHANUM.sub("", unique).lower(),
        "unique_upper_alnum": PATTERN_ALPHANUM.sub("", unique).upper(),
        "timestamp": relative_time(freezed_time, False),
        "timeISO": relative_time(freezed_time, True),
    }

    yield ctx


@pytest.fixture(scope="session")
def record_mode(request):
    """Manage compatibility with DD client libraries."""
    return {"false": "none", "true": "rewrite", "none": "new_episodes"}[RECORD]


def _disable_recording():
    """Disable VCR.py integration."""
    return RECORD == "none"


@pytest.fixture(scope="session")
def disable_recording(request):
    """Disable VCR.py integration. This overrides a pytest-recording fixture."""
    return _disable_recording()


@pytest.fixture
def vcr_config():
    config = dict(
        filter_headers=(
            "DD-API-KEY",
            "DD-APPLICATION-KEY",
            "User-Agent",
            "Accept-Encoding",
        ),
        match_on=[
            "method",
            "scheme",
            "host",
            "port",
            "path",
            "query",
            "body",
            "headers",
        ],
    )
    if tracer:
        from urllib.parse import urlparse

        config["ignore_hosts"] = [urlparse(tracer._writer.agent_url).hostname]

    return config


@pytest.fixture
def default_cassette_name(default_cassette_name):
    return PATTERN_DOUBLE_UNDERSCORE.sub("_", default_cassette_name)


@pytest.fixture
def freezed_time(default_cassette_name, record_mode, vcr):
    from dateutil import parser

    if record_mode in {"new_episodes", "rewrite"}:
        tzinfo = datetime.now().astimezone().tzinfo
        freeze_at = datetime.now().replace(tzinfo=tzinfo).isoformat()
        if record_mode == "rewrite":
            pathlib.Path(vcr._path).parent.mkdir(parents=True, exist_ok=True)
            with pathlib.Path(vcr._path).with_suffix(".frozen").open("w+") as f:
                f.write(freeze_at)
    else:
        freeze_file = pathlib.Path(vcr._path).with_suffix(".frozen")
        if not freeze_file.exists():
            msg = (
                "Time file '{}' not found: create one setting `RECORD=true` or "
                "ignore it using `RECORD=none`".format(freeze_file)
            )
            raise RuntimeError(msg)
        with freeze_file.open("r") as f:
            freeze_at = f.readline().strip()

        if not pathlib.Path(vcr._path).exists():
            msg = (
                "Cassette '{}' not found: create one setting `RECORD=true` or "
                "ignore it using `RECORD=none`".format(vcr._path)
            )
            raise RuntimeError(msg)

    return parser.isoparse(freeze_at)


def pytest_recording_configure(config, vcr):
    from vcr import matchers
    from vcr.util import read_body

    is_text_json = matchers._header_checker("text/json")
    transformer = matchers._transform_json

    def body(r1, r2):
        if is_text_json(r1.headers) and is_text_json(r2.headers):
            assert transformer(read_body(r1)) == transformer(read_body(r2))
        else:
            matchers.body(r1, r2)

    vcr.matchers["body"] = body


@given('a valid "apiKeyAuth" key in the system')
def a_valid_api_key(configuration):
    """a valid API key."""
    configuration.api_key["apiKeyAuth"] = os.getenv("DD_TEST_CLIENT_API_KEY", "fake")


@given('a valid "appKeyAuth" key in the system')
def a_valid_application_key(configuration):
    """a valid Application key."""
    configuration.api_key["appKeyAuth"] = os.getenv("DD_TEST_CLIENT_APP_KEY", "fake")


@pytest.fixture(scope="module")
def package_name(api_version):
    return "datadog_api_client." + api_version


@pytest.fixture(scope="module")
def undo_operations():
    result = {}
    for f in pathlib.Path(os.path.dirname(__file__)).rglob("undo.json"):
        version = f.parent.parent.name
        with f.open() as fp:
            data = json.load(fp)
            result[version] = {
                snake_case(operation_id): settings.get("undo") for operation_id, settings in data.items()
            }

    return result


def build_configuration():
    c = Configuration(return_http_data_only=False, spec_property_naming=True)
    c.connection_pool_maxsize = 0
    c.debug = debug = os.getenv("DEBUG") in {"true", "1", "yes", "on"}
    if debug:  # enable vcr logs for DEBUG=true
        vcr_log = logging.getLogger("vcr")
        vcr_log.setLevel(logging.INFO)
    if "DD_TEST_SITE" in os.environ:
        c.server_index = 2
        c.server_variables["site"] = os.environ["DD_TEST_SITE"]
    return c


@pytest.fixture
def configuration():
    return build_configuration()


@pytest.fixture
def client(configuration):
    with ApiClient(configuration) as api_client:
        yield api_client


@given(parsers.parse('an instance of "{name}" API'))
def api(context, package_name, client, name):
    """Return an API instance."""
    module_name = snake_case(name)
    package = importlib.import_module(f"{package_name}.api.{module_name}_api")
    context["api"] = {
        "api": getattr(package, name + "Api")(client),
        "package": package_name,
        "calls": [],
    }


@given(parsers.parse('operation "{name}" enabled'))
def operation_enabled(client, name):
    """Enable the unstable operation specific in the clause."""
    client.configuration.unstable_operations[snake_case(name)] = True


@given(parsers.parse('new "{name}" request'))
def api_request(configuration, context, name):
    """Call an endpoint."""
    api = context["api"]
    context["api_request"] = {
        "api": api["api"],
        "request": getattr(api["api"], snake_case(name)),
        "args": [],
        "kwargs": {},
        "response": (None, None, None),
    }


@given(parsers.parse("body with value {data}"))
def request_body(context, data):
    """Set request body."""
    tpl = Template(data).render(**context)
    context["api_request"]["kwargs"]["body"] = tpl


@given(parsers.parse('body from file "{path}"'))
def request_body_from_file(context, path, package_name):
    """Set request body."""
    version = package_name.split(".")[-1]
    with open(os.path.join(os.path.dirname(__file__), version, "features", path)) as f:
        data = f.read()
    tpl = Template(data).render(**context)
    context["api_request"]["kwargs"]["body"] = tpl


@given(parsers.parse('request contains "{name}" parameter from "{path}"'))
def request_parameter(context, name, path):
    """Set request parameter."""
    context["api_request"]["kwargs"][escape_reserved_keyword(snake_case(name))] = json.dumps(glom(context, path))


@given(parsers.parse('request contains "{name}" parameter with value {value}'))
def request_parameter_with_value(context, name, value):
    """Set request parameter."""
    tpl = Template(value).render(**context)
    context["api_request"]["kwargs"][escape_reserved_keyword(snake_case(name))] = tpl


def build_given(version, operation):
    @sleep_after_request
    def wrapper(context, undo):
        name = operation["tag"].replace(" ", "")
        module_name = snake_case(operation["tag"])
        operation_name = snake_case(operation["operationId"])
        package_name = f"datadog_api_client.{version}"

        # make sure we have a fresh instance of API client and configuration
        configuration = build_configuration()
        configuration.api_key["apiKeyAuth"] = os.getenv("DD_TEST_CLIENT_API_KEY", "fake")
        configuration.api_key["appKeyAuth"] = os.getenv("DD_TEST_CLIENT_APP_KEY", "fake")
        configuration.check_input_type = False
        configuration.return_http_data_only = True

        # enable unstable operation
        if operation_name in configuration.unstable_operations:
            configuration.unstable_operations[operation_name] = True

        package = importlib.import_module(f"{package_name}.api.{module_name}_api")
        with ApiClient(configuration) as client:
            api = getattr(package, name + "Api")(client)
            operation_method = getattr(api, operation_name)

            # perform operation
            def build_param(p):
                if "value" in p:
                    return json.loads(Template(p["value"]).render(**context))
                if "source" in p:
                    return glom(context, p["source"])

            kwargs = {
                escape_reserved_keyword(snake_case(p["name"])): build_param(p) for p in operation.get("parameters", [])
            }
            result = operation_method(**kwargs)

            # register undo method
            def undo_operation():
                return undo(api, version, operation_name, result, client=client)

            if tracer:
                undo_operation = tracer.wrap(name="undo", resource=operation["step"])(undo_operation)

            context["undo_operations"].append(undo_operation)

            # optional re-shaping
            if "source" in operation:
                result = glom(result, operation["source"])

            # store response in fixtures
            context[operation["key"]] = result

    return wrapper


for f in pathlib.Path(os.path.dirname(__file__)).rglob("given.json"):
    version = f.parent.parent.name
    with f.open() as fp:
        for settings in json.load(fp):
            given(settings["step"])(build_given(version, settings))


@pytest.fixture
def undo(package_name, undo_operations, client):
    """Clean after operation."""

    def cleanup(api, version, operation_id, response, client=client):
        operation = undo_operations.get(version, {}).get(operation_id)
        if operation_id is None:
            raise NotImplementedError((version, operation_id))

        if operation["type"] is None:
            raise NotImplementedError((version, operation_id))

        if operation["type"] != "unsafe":
            return

        operation_name = snake_case(operation["operationId"])
        method = getattr(api, operation_name)
        kwargs = {}
        for parameter in operation.get("parameters", []):
            if "source" in parameter:
                kwargs[parameter["name"]] = glom(response, parameter["source"])
            elif "template" in parameter:
                variables = meta.find_undeclared_variables(Environment().parse(parameter["template"]))
                ctx = {}
                for var in variables:
                    ctx[var] = glom(response, var)
                kwargs[parameter["name"]] = json.loads(Template(parameter["template"]).render(**ctx))

        if operation_name in client.configuration.unstable_operations:
            client.configuration.unstable_operations[operation_name] = True

        try:
            method(**kwargs)
        except exceptions.ApiException as e:
            warnings.warn(f"failed undo: {e}")

    yield cleanup


@when("the request is sent")
def execute_request(undo, context, client, api_version):
    """Execute the prepared request."""
    api_request = context["api_request"]

    params_map = getattr(api_request["api"], f'_{api_request["request"].__name__}_endpoint').params_map
    for k, v in api_request["kwargs"].items():
        api_request["kwargs"][k] = client.deserialize(v, params_map[k]["openapi_types"], True)

    try:
        response = api_request["request"](*api_request["args"], **api_request["kwargs"])
        # Reserialise the response body to JSON to facilitate test assertions
        response_body_json = client.sanitize_for_serialization(response[0])
        api_request["response"] = [response_body_json, response[1], response[2]]
    except exceptions.ApiException as e:
        # If we have an exception, make a stub response object to use for assertions
        # Instead of finding the response class of the method, we use the fact that all
        # responses returned have an ordered response of body|status|headers
        api_request["response"] = [e.body, e.status, e.headers]
        return

    api = api_request["api"]
    operation_id = api_request["request"].__name__
    response = api_request["response"][0]

    def undo_operation():
        return undo(api, api_version, operation_id, response)

    if tracer:
        undo_operation = tracer.wrap(name="undo", resource="execute request")(undo_operation)

    context["undo_operations"].append(undo_operation)


@when("the request with pagination is sent")
def execute_request_with_pagination(undo, context, client, api_version):
    """Execute the prepared paginated request."""
    api_request = context["api_request"]

    params_map = getattr(api_request["api"], f'_{api_request["request"].__name__}_endpoint').params_map
    for k, v in api_request["kwargs"].items():
        api_request["kwargs"][k] = client.deserialize(v, params_map[k]["openapi_types"], True)

    kwargs = api_request["kwargs"]
    client.configuration.return_http_data_only = True
    method = getattr(api_request["api"], f"{api_request['request'].__name__}_with_pagination")
    try:
        response = list(method(*api_request["args"], **kwargs))
        # Reserialise the response body to JSON to facilitate test assertions
        response_body_json = client.sanitize_for_serialization(response)
        api_request["response"] = [response_body_json, 200, None]
    except exceptions.ApiException as e:
        # If we have an exception, make a stub response object to use for assertions
        # Instead of finding the response class of the method, we use the fact that all
        # responses returned have an ordered response of body|status|headers
        api_request["response"] = [e.body, e.status, e.headers]
    finally:
        client.configuration.return_http_data_only = False


@then(parsers.parse("the response status is {status:d} {description}"))
def the_status_is(context, status, description):
    """Check the status."""
    assert status == context["api_request"]["response"][1]


@then(parsers.parse('the response "{response_path}" is equal to {value}'))
def expect_equal(context, response_path, value):
    response_value = glom(context["api_request"]["response"][0], response_path)
    test_value = json.loads(Template(value).render(**context))
    assert test_value == response_value


@then(parsers.parse('the response "{response_path}" has the same value as "{fixture_path}"'))
def expect_equal_value(context, response_path, fixture_path):
    fixture_value = glom(context, fixture_path)
    response_value = glom(context["api_request"]["response"][0], response_path)
    assert fixture_value == response_value


@then(parsers.parse('the response "{response_path}" has length {fixture_length:d}'))
def expect_equal_length(context, response_path, fixture_length):
    response_value = glom(context["api_request"]["response"][0], response_path)
    assert fixture_length == len(response_value)


@then(parsers.parse("the response has {fixture_length:d} items"))
def expect_equal_response_items(context, fixture_length):
    response = context["api_request"]["response"][0]
    assert fixture_length == len(response)


@then(parsers.parse('the response "{response_path}" is false'))
def expect_false(context, response_path):
    response_value = glom(context["api_request"]["response"][0], response_path)
    assert not response_value



"""
Get all Cloud Workload Security Agent rules returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.cloud_workload_security_api import CloudWorkloadSecurityApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = CloudWorkloadSecurityApi(api_client)
    response = api_instance.list_cloud_workload_security_agent_rules()

    print(response)



"""
Update a role returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.roles_api import RolesApi
from datadog_api_client.v2.model.permissions_type import PermissionsType
from datadog_api_client.v2.model.relationship_to_permission_data import RelationshipToPermissionData
from datadog_api_client.v2.model.relationship_to_permissions import RelationshipToPermissions
from datadog_api_client.v2.model.role_relationships import RoleRelationships
from datadog_api_client.v2.model.role_update_attributes import RoleUpdateAttributes
from datadog_api_client.v2.model.role_update_data import RoleUpdateData
from datadog_api_client.v2.model.role_update_request import RoleUpdateRequest
from datadog_api_client.v2.model.roles_type import RolesType

# there is a valid "role" in the system
ROLE_DATA_ATTRIBUTES_NAME = environ["ROLE_DATA_ATTRIBUTES_NAME"]
ROLE_DATA_ID = environ["ROLE_DATA_ID"]

# there is a valid "permission" in the system
PERMISSION_ID = environ["PERMISSION_ID"]

body = RoleUpdateRequest(
    data=RoleUpdateData(
        id=ROLE_DATA_ID,
        type=RolesType.ROLES,
        attributes=RoleUpdateAttributes(
            name="developers-updated",
        ),
        relationships=RoleRelationships(
            permissions=RelationshipToPermissions(
                data=[
                    RelationshipToPermissionData(
                        id=PERMISSION_ID,
                        type=PermissionsType.PERMISSIONS,
                    ),
                ],
            ),
        ),
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = RolesApi(api_client)
    response = api_instance.update_role(role_id=ROLE_DATA_ID, body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    none_type,
    unset,
    UnsetType,
)


class MonitorThresholds(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "critical": (float,),
            "critical_recovery": (float, none_type),
            "ok": (float, none_type),
            "unknown": (float, none_type),
            "warning": (float, none_type),
            "warning_recovery": (float, none_type),
        }

    attribute_map = {
        "critical": "critical",
        "critical_recovery": "critical_recovery",
        "ok": "ok",
        "unknown": "unknown",
        "warning": "warning",
        "warning_recovery": "warning_recovery",
    }

    def __init__(
        self_,
        critical: Union[float, UnsetType] = unset,
        critical_recovery: Union[float, none_type, UnsetType] = unset,
        ok: Union[float, none_type, UnsetType] = unset,
        unknown: Union[float, none_type, UnsetType] = unset,
        warning: Union[float, none_type, UnsetType] = unset,
        warning_recovery: Union[float, none_type, UnsetType] = unset,
        **kwargs,
    ):
        """
        List of the different monitor threshold available.

        :param critical: The monitor ``CRITICAL`` threshold.
        :type critical: float, optional

        :param critical_recovery: The monitor ``CRITICAL`` recovery threshold.
        :type critical_recovery: float, none_type, optional

        :param ok: The monitor ``OK`` threshold.
        :type ok: float, none_type, optional

        :param unknown: The monitor UNKNOWN threshold.
        :type unknown: float, none_type, optional

        :param warning: The monitor ``WARNING`` threshold.
        :type warning: float, none_type, optional

        :param warning_recovery: The monitor ``WARNING`` recovery threshold.
        :type warning_recovery: float, none_type, optional
        """
        if critical is not unset:
            kwargs["critical"] = critical
        if critical_recovery is not unset:
            kwargs["critical_recovery"] = critical_recovery
        if ok is not unset:
            kwargs["ok"] = ok
        if unknown is not unset:
            kwargs["unknown"] = unknown
        if warning is not unset:
            kwargs["warning"] = warning
        if warning_recovery is not unset:
            kwargs["warning_recovery"] = warning_recovery
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.organization_billing import OrganizationBilling
    from datadog_api_client.v1.model.organization_subscription import OrganizationSubscription


class OrganizationCreateBody(ModelNormal):
    validations = {
        "name": {
            "max_length": 32,
        },
    }

    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.organization_billing import OrganizationBilling
        from datadog_api_client.v1.model.organization_subscription import OrganizationSubscription

        return {
            "billing": (OrganizationBilling,),
            "name": (str,),
            "subscription": (OrganizationSubscription,),
        }

    attribute_map = {
        "billing": "billing",
        "name": "name",
        "subscription": "subscription",
    }

    def __init__(
        self_,
        name: str,
        billing: Union[OrganizationBilling, UnsetType] = unset,
        subscription: Union[OrganizationSubscription, UnsetType] = unset,
        **kwargs,
    ):
        """
        Object describing an organization to create.

        :param billing: A JSON array of billing type. **Deprecated**.
        :type billing: OrganizationBilling, optional

        :param name: The name of the new child-organization, limited to 32 characters.
        :type name: str

        :param subscription: Subscription definition. **Deprecated**.
        :type subscription: OrganizationSubscription, optional
        """
        if billing is not unset:
            kwargs["billing"] = billing
        if subscription is not unset:
            kwargs["subscription"] = subscription
        super().__init__(kwargs)

        self_.name = name



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class DashboardType(ModelSimple):
    """
    The type of the dashboard.

    :param value: Must be one of ["custom_timeboard", "custom_screenboard", "integration_screenboard", "integration_timeboard", "host_timeboard"].
    :type value: str
    """

    allowed_values = {
        "custom_timeboard",
        "custom_screenboard",
        "integration_screenboard",
        "integration_timeboard",
        "host_timeboard",
    }
    CUSTOM_TIMEBOARD: ClassVar["DashboardType"]
    CUSTOM_SCREENBOARD: ClassVar["DashboardType"]
    INTEGRATION_SCREENBOARD: ClassVar["DashboardType"]
    INTEGRATION_TIMEBOARD: ClassVar["DashboardType"]
    HOST_TIMEBOARD: ClassVar["DashboardType"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


DashboardType.CUSTOM_TIMEBOARD = DashboardType("custom_timeboard")
DashboardType.CUSTOM_SCREENBOARD = DashboardType("custom_screenboard")
DashboardType.INTEGRATION_SCREENBOARD = DashboardType("integration_screenboard")
DashboardType.INTEGRATION_TIMEBOARD = DashboardType("integration_timeboard")
DashboardType.HOST_TIMEBOARD = DashboardType("host_timeboard")



"""
Get a single service object returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.pager_duty_integration_api import PagerDutyIntegrationApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = PagerDutyIntegrationApi(api_client)
    response = api_instance.get_pager_duty_integration_service(
        service_name="service_name",
    )

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.incident_update_attributes import IncidentUpdateAttributes
    from datadog_api_client.v2.model.incident_update_relationships import IncidentUpdateRelationships
    from datadog_api_client.v2.model.incident_type import IncidentType


class IncidentUpdateData(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.incident_update_attributes import IncidentUpdateAttributes
        from datadog_api_client.v2.model.incident_update_relationships import IncidentUpdateRelationships
        from datadog_api_client.v2.model.incident_type import IncidentType

        return {
            "attributes": (IncidentUpdateAttributes,),
            "id": (str,),
            "relationships": (IncidentUpdateRelationships,),
            "type": (IncidentType,),
        }

    attribute_map = {
        "attributes": "attributes",
        "id": "id",
        "relationships": "relationships",
        "type": "type",
    }

    def __init__(
        self_,
        id: str,
        type: IncidentType,
        attributes: Union[IncidentUpdateAttributes, UnsetType] = unset,
        relationships: Union[IncidentUpdateRelationships, UnsetType] = unset,
        **kwargs,
    ):
        """
        Incident data for an update request.

        :param attributes: The incident's attributes for an update request.
        :type attributes: IncidentUpdateAttributes, optional

        :param id: The team's ID.
        :type id: str

        :param relationships: The incident's relationships for an update request.
        :type relationships: IncidentUpdateRelationships, optional

        :param type: Incident resource type.
        :type type: IncidentType
        """
        if attributes is not unset:
            kwargs["attributes"] = attributes
        if relationships is not unset:
            kwargs["relationships"] = relationships
        super().__init__(kwargs)

        self_.id = id
        self_.type = type



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class IntakePayloadAccepted(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "errors": ([str],),
        }

    attribute_map = {
        "errors": "errors",
    }

    def __init__(self_, errors: Union[List[str], UnsetType] = unset, **kwargs):
        """
        The payload accepted for intake.

        :param errors: A list of errors.
        :type errors: [str], optional
        """
        if errors is not unset:
            kwargs["errors"] = errors
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class SyntheticsTestOptionsRetry(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "count": (int,),
            "interval": (float,),
        }

    attribute_map = {
        "count": "count",
        "interval": "interval",
    }

    def __init__(self_, count: Union[int, UnsetType] = unset, interval: Union[float, UnsetType] = unset, **kwargs):
        """
        Object describing the retry strategy to apply to a Synthetic test.

        :param count: Number of times a test needs to be retried before marking a
            location as failed. Defaults to 0.
        :type count: int, optional

        :param interval: Time interval between retries (in milliseconds). Defaults to
            300ms.
        :type interval: float, optional
        """
        if count is not unset:
            kwargs["count"] = count
        if interval is not unset:
            kwargs["interval"] = interval
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Any, Dict, Union

from datadog_api_client.api_client import ApiClient, Endpoint as _Endpoint
from datadog_api_client.v2.model.service_definitions_list_response import ServiceDefinitionsListResponse
from datadog_api_client.v2.model.service_definition_create_response import ServiceDefinitionCreateResponse
from datadog_api_client.v2.model.service_definitions_create_request import ServiceDefinitionsCreateRequest
from datadog_api_client.v2.model.service_definition_v2 import ServiceDefinitionV2
from datadog_api_client.v2.model.service_definition_get_response import ServiceDefinitionGetResponse


class ServiceDefinitionApi:
    """
    API to create, update, retrieve and delete service definitions.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self._create_or_update_service_definitions_endpoint = _Endpoint(
            settings={
                "response_type": (ServiceDefinitionCreateResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v2/services/definitions",
                "operation_id": "create_or_update_service_definitions",
                "http_method": "POST",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (ServiceDefinitionsCreateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._delete_service_definition_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v2/services/definitions/{service_name}",
                "operation_id": "delete_service_definition",
                "http_method": "DELETE",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "service_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "service_name",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["*/*"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._get_service_definition_endpoint = _Endpoint(
            settings={
                "response_type": (ServiceDefinitionGetResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v2/services/definitions/{service_name}",
                "operation_id": "get_service_definition",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "service_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "service_name",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_service_definitions_endpoint = _Endpoint(
            settings={
                "response_type": (ServiceDefinitionsListResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v2/services/definitions",
                "operation_id": "list_service_definitions",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={},
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

    def create_or_update_service_definitions(
        self,
        body: Union[ServiceDefinitionsCreateRequest, ServiceDefinitionV2, str],
    ) -> ServiceDefinitionCreateResponse:
        """Create or update service definition.

        Create or update service definition in the Datadog Service Catalog.

        :param body: Service Definition YAML/JSON.
        :type body: ServiceDefinitionsCreateRequest
        :rtype: ServiceDefinitionCreateResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._create_or_update_service_definitions_endpoint.call_with_http_info(**kwargs)

    def delete_service_definition(
        self,
        service_name: str,
    ) -> None:
        """Delete a single service definition.

        Delete a single service definition in the Datadog Service Catalog.

        :param service_name: The name of the service.
        :type service_name: str
        :rtype: None
        """
        kwargs: Dict[str, Any] = {}
        kwargs["service_name"] = service_name

        return self._delete_service_definition_endpoint.call_with_http_info(**kwargs)

    def get_service_definition(
        self,
        service_name: str,
    ) -> ServiceDefinitionGetResponse:
        """Get a single service definition.

        Get a single service definition from the Datadog Service Catalog.

        :param service_name: The name of the service.
        :type service_name: str
        :rtype: ServiceDefinitionGetResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["service_name"] = service_name

        return self._get_service_definition_endpoint.call_with_http_info(**kwargs)

    def list_service_definitions(
        self,
    ) -> ServiceDefinitionsListResponse:
        """Get all service definitions.

        Get a list of all service definitions from the Datadog Service Catalog.

        :rtype: ServiceDefinitionsListResponse
        """
        kwargs: Dict[str, Any] = {}
        return self._list_service_definitions_endpoint.call_with_http_info(**kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class WidgetLiveSpan(ModelSimple):
    """
    The available timeframes depend on the widget you are using.

    :param value: Must be one of ["1m", "5m", "10m", "15m", "30m", "1h", "4h", "1d", "2d", "1w", "1mo", "3mo", "6mo", "1y", "alert"].
    :type value: str
    """

    allowed_values = {
        "1m",
        "5m",
        "10m",
        "15m",
        "30m",
        "1h",
        "4h",
        "1d",
        "2d",
        "1w",
        "1mo",
        "3mo",
        "6mo",
        "1y",
        "alert",
    }
    PAST_ONE_MINUTE: ClassVar["WidgetLiveSpan"]
    PAST_FIVE_MINUTES: ClassVar["WidgetLiveSpan"]
    PAST_TEN_MINUTES: ClassVar["WidgetLiveSpan"]
    PAST_FIFTEEN_MINUTES: ClassVar["WidgetLiveSpan"]
    PAST_THIRTY_MINUTES: ClassVar["WidgetLiveSpan"]
    PAST_ONE_HOUR: ClassVar["WidgetLiveSpan"]
    PAST_FOUR_HOURS: ClassVar["WidgetLiveSpan"]
    PAST_ONE_DAY: ClassVar["WidgetLiveSpan"]
    PAST_TWO_DAYS: ClassVar["WidgetLiveSpan"]
    PAST_ONE_WEEK: ClassVar["WidgetLiveSpan"]
    PAST_ONE_MONTH: ClassVar["WidgetLiveSpan"]
    PAST_THREE_MONTHS: ClassVar["WidgetLiveSpan"]
    PAST_SIX_MONTHS: ClassVar["WidgetLiveSpan"]
    PAST_ONE_YEAR: ClassVar["WidgetLiveSpan"]
    ALERT: ClassVar["WidgetLiveSpan"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


WidgetLiveSpan.PAST_ONE_MINUTE = WidgetLiveSpan("1m")
WidgetLiveSpan.PAST_FIVE_MINUTES = WidgetLiveSpan("5m")
WidgetLiveSpan.PAST_TEN_MINUTES = WidgetLiveSpan("10m")
WidgetLiveSpan.PAST_FIFTEEN_MINUTES = WidgetLiveSpan("15m")
WidgetLiveSpan.PAST_THIRTY_MINUTES = WidgetLiveSpan("30m")
WidgetLiveSpan.PAST_ONE_HOUR = WidgetLiveSpan("1h")
WidgetLiveSpan.PAST_FOUR_HOURS = WidgetLiveSpan("4h")
WidgetLiveSpan.PAST_ONE_DAY = WidgetLiveSpan("1d")
WidgetLiveSpan.PAST_TWO_DAYS = WidgetLiveSpan("2d")
WidgetLiveSpan.PAST_ONE_WEEK = WidgetLiveSpan("1w")
WidgetLiveSpan.PAST_ONE_MONTH = WidgetLiveSpan("1mo")
WidgetLiveSpan.PAST_THREE_MONTHS = WidgetLiveSpan("3mo")
WidgetLiveSpan.PAST_SIX_MONTHS = WidgetLiveSpan("6mo")
WidgetLiveSpan.PAST_ONE_YEAR = WidgetLiveSpan("1y")
WidgetLiveSpan.ALERT = WidgetLiveSpan("alert")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.synthetics_global_variable_attributes import SyntheticsGlobalVariableAttributes
    from datadog_api_client.v1.model.synthetics_global_variable_parse_test_options import (
        SyntheticsGlobalVariableParseTestOptions,
    )
    from datadog_api_client.v1.model.synthetics_global_variable_value import SyntheticsGlobalVariableValue


class SyntheticsGlobalVariable(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.synthetics_global_variable_attributes import SyntheticsGlobalVariableAttributes
        from datadog_api_client.v1.model.synthetics_global_variable_parse_test_options import (
            SyntheticsGlobalVariableParseTestOptions,
        )
        from datadog_api_client.v1.model.synthetics_global_variable_value import SyntheticsGlobalVariableValue

        return {
            "attributes": (SyntheticsGlobalVariableAttributes,),
            "description": (str,),
            "id": (str,),
            "name": (str,),
            "parse_test_options": (SyntheticsGlobalVariableParseTestOptions,),
            "parse_test_public_id": (str,),
            "tags": ([str],),
            "value": (SyntheticsGlobalVariableValue,),
        }

    attribute_map = {
        "attributes": "attributes",
        "description": "description",
        "id": "id",
        "name": "name",
        "parse_test_options": "parse_test_options",
        "parse_test_public_id": "parse_test_public_id",
        "tags": "tags",
        "value": "value",
    }
    read_only_vars = {
        "id",
    }

    def __init__(
        self_,
        description: str,
        name: str,
        tags: List[str],
        value: SyntheticsGlobalVariableValue,
        attributes: Union[SyntheticsGlobalVariableAttributes, UnsetType] = unset,
        id: Union[str, UnsetType] = unset,
        parse_test_options: Union[SyntheticsGlobalVariableParseTestOptions, UnsetType] = unset,
        parse_test_public_id: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        Synthetics global variable.

        :param attributes: Attributes of the global variable.
        :type attributes: SyntheticsGlobalVariableAttributes, optional

        :param description: Description of the global variable.
        :type description: str

        :param id: Unique identifier of the global variable.
        :type id: str, optional

        :param name: Name of the global variable. Unique across Synthetics global variables.
        :type name: str

        :param parse_test_options: Parser options to use for retrieving a Synthetics global variable from a Synthetics Test. Used in conjunction with ``parse_test_public_id``.
        :type parse_test_options: SyntheticsGlobalVariableParseTestOptions, optional

        :param parse_test_public_id: A Synthetic test ID to use as a test to generate the variable value.
        :type parse_test_public_id: str, optional

        :param tags: Tags of the global variable.
        :type tags: [str]

        :param value: Value of the global variable.
        :type value: SyntheticsGlobalVariableValue
        """
        if attributes is not unset:
            kwargs["attributes"] = attributes
        if id is not unset:
            kwargs["id"] = id
        if parse_test_options is not unset:
            kwargs["parse_test_options"] = parse_test_options
        if parse_test_public_id is not unset:
            kwargs["parse_test_public_id"] = parse_test_public_id
        super().__init__(kwargs)

        self_.description = description
        self_.name = name
        self_.tags = tags
        self_.value = value



"""
Get a list of pipelines events returns "OK" response
"""

from datetime import datetime
from dateutil.relativedelta import relativedelta
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.ci_visibility_pipelines_api import CIVisibilityPipelinesApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = CIVisibilityPipelinesApi(api_client)
    response = api_instance.list_ci_app_pipeline_events(
        filter_query="@ci.provider.name:circleci",
        filter_from=(datetime.now() + relativedelta(minutes=-30)),
        filter_to=datetime.now(),
        page_limit=5,
    )

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class RUMWarning(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "code": (str,),
            "detail": (str,),
            "title": (str,),
        }

    attribute_map = {
        "code": "code",
        "detail": "detail",
        "title": "title",
    }

    def __init__(
        self_,
        code: Union[str, UnsetType] = unset,
        detail: Union[str, UnsetType] = unset,
        title: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        A warning message indicating something that went wrong with the query.

        :param code: A unique code for this type of warning.
        :type code: str, optional

        :param detail: A detailed explanation of this specific warning.
        :type detail: str, optional

        :param title: A short human-readable summary of the warning.
        :type title: str, optional
        """
        if code is not unset:
            kwargs["code"] = code
        if detail is not unset:
            kwargs["detail"] = detail
        if title is not unset:
            kwargs["title"] = title
        super().__init__(kwargs)



"""
Search logs returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.logs_api import LogsApi
from datadog_api_client.v1.model.logs_list_request import LogsListRequest
from datadog_api_client.v1.model.logs_list_request_time import LogsListRequestTime
from datadog_api_client.v1.model.logs_sort import LogsSort
from datetime import datetime
from dateutil.tz import tzutc

body = LogsListRequest(
    index="retention-3,retention-15",
    query="service:web* AND @http.status_code:[200 TO 299]",
    sort=LogsSort.TIME_ASCENDING,
    time=LogsListRequestTime(
        _from=datetime(2020, 2, 2, 2, 2, 2, 202000, tzinfo=tzutc()),
        to=datetime(2020, 2, 20, 2, 2, 2, 202000, tzinfo=tzutc()),
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = LogsApi(api_client)
    response = api_instance.list_logs(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.synthetics_basic_auth_digest_type import SyntheticsBasicAuthDigestType


class SyntheticsBasicAuthDigest(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.synthetics_basic_auth_digest_type import SyntheticsBasicAuthDigestType

        return {
            "password": (str,),
            "type": (SyntheticsBasicAuthDigestType,),
            "username": (str,),
        }

    attribute_map = {
        "password": "password",
        "type": "type",
        "username": "username",
    }

    def __init__(
        self_, password: str, username: str, type: Union[SyntheticsBasicAuthDigestType, UnsetType] = unset, **kwargs
    ):
        """
        Object to handle digest authentication when performing the test.

        :param password: Password to use for the digest authentication.
        :type password: str

        :param type: The type of basic authentication to use when performing the test.
        :type type: SyntheticsBasicAuthDigestType, optional

        :param username: Username to use for the digest authentication.
        :type username: str
        """
        if type is not unset:
            kwargs["type"] = type
        super().__init__(kwargs)

        self_.password = password
        self_.username = username



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class WidgetGrouping(ModelSimple):
    """
    The kind of grouping to use.

    :param value: Must be one of ["check", "cluster"].
    :type value: str
    """

    allowed_values = {
        "check",
        "cluster",
    }
    CHECK: ClassVar["WidgetGrouping"]
    CLUSTER: ClassVar["WidgetGrouping"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


WidgetGrouping.CHECK = WidgetGrouping("check")
WidgetGrouping.CLUSTER = WidgetGrouping("cluster")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class SecurityMonitoringRuleNewValueOptionsLearningDuration(ModelSimple):
    """
    The duration in days during which values are learned, and after which signals will be generated for values that
        weren't learned. If set to 0, a signal will be generated for all new values after the first value is learned.

    :param value: If omitted defaults to 0. Must be one of [0, 1, 7].
    :type value: int
    """

    allowed_values = {
        0,
        1,
        7,
    }
    ZERO_DAYS: ClassVar["SecurityMonitoringRuleNewValueOptionsLearningDuration"]
    ONE_DAY: ClassVar["SecurityMonitoringRuleNewValueOptionsLearningDuration"]
    SEVEN_DAYS: ClassVar["SecurityMonitoringRuleNewValueOptionsLearningDuration"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (int,),
        }


SecurityMonitoringRuleNewValueOptionsLearningDuration.ZERO_DAYS = SecurityMonitoringRuleNewValueOptionsLearningDuration(
    0
)
SecurityMonitoringRuleNewValueOptionsLearningDuration.ONE_DAY = SecurityMonitoringRuleNewValueOptionsLearningDuration(1)
SecurityMonitoringRuleNewValueOptionsLearningDuration.SEVEN_DAYS = (
    SecurityMonitoringRuleNewValueOptionsLearningDuration(7)
)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class SyntheticsBrowserVariableType(ModelSimple):
    """
    Type of browser test variable.

    :param value: Must be one of ["element", "email", "global", "javascript", "text"].
    :type value: str
    """

    allowed_values = {
        "element",
        "email",
        "global",
        "javascript",
        "text",
    }
    ELEMENT: ClassVar["SyntheticsBrowserVariableType"]
    EMAIL: ClassVar["SyntheticsBrowserVariableType"]
    GLOBAL: ClassVar["SyntheticsBrowserVariableType"]
    JAVASCRIPT: ClassVar["SyntheticsBrowserVariableType"]
    TEXT: ClassVar["SyntheticsBrowserVariableType"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


SyntheticsBrowserVariableType.ELEMENT = SyntheticsBrowserVariableType("element")
SyntheticsBrowserVariableType.EMAIL = SyntheticsBrowserVariableType("email")
SyntheticsBrowserVariableType.GLOBAL = SyntheticsBrowserVariableType("global")
SyntheticsBrowserVariableType.JAVASCRIPT = SyntheticsBrowserVariableType("javascript")
SyntheticsBrowserVariableType.TEXT = SyntheticsBrowserVariableType("text")



"""
Edit a global variable returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.synthetics_api import SyntheticsApi
from datadog_api_client.v1.model.synthetics_global_variable import SyntheticsGlobalVariable
from datadog_api_client.v1.model.synthetics_global_variable_attributes import SyntheticsGlobalVariableAttributes
from datadog_api_client.v1.model.synthetics_global_variable_parse_test_options import (
    SyntheticsGlobalVariableParseTestOptions,
)
from datadog_api_client.v1.model.synthetics_global_variable_parse_test_options_type import (
    SyntheticsGlobalVariableParseTestOptionsType,
)
from datadog_api_client.v1.model.synthetics_global_variable_parser_type import SyntheticsGlobalVariableParserType
from datadog_api_client.v1.model.synthetics_global_variable_value import SyntheticsGlobalVariableValue
from datadog_api_client.v1.model.synthetics_restricted_roles import SyntheticsRestrictedRoles
from datadog_api_client.v1.model.synthetics_variable_parser import SyntheticsVariableParser

body = SyntheticsGlobalVariable(
    attributes=SyntheticsGlobalVariableAttributes(
        restricted_roles=SyntheticsRestrictedRoles(
            [
                "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            ]
        ),
    ),
    description="Example description",
    name="MY_VARIABLE",
    parse_test_options=SyntheticsGlobalVariableParseTestOptions(
        field="content-type",
        local_variable_name="LOCAL_VARIABLE",
        parser=SyntheticsVariableParser(
            type=SyntheticsGlobalVariableParserType.REGEX,
            value=".*",
        ),
        type=SyntheticsGlobalVariableParseTestOptionsType.HTTP_BODY,
    ),
    parse_test_public_id="abc-def-123",
    tags=[
        "team:front",
        "test:workflow-1",
    ],
    value=SyntheticsGlobalVariableValue(
        secure=True,
        value="value",
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = SyntheticsApi(api_client)
    response = api_instance.edit_global_variable(variable_id="variable_id", body=body)

    print(response)



"""
Get estimated cost across your account returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.usage_metering_api import UsageMeteringApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = UsageMeteringApi(api_client)
    response = api_instance.get_estimated_cost_by_org()

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.dashboard_list import DashboardList


class DashboardListListResponse(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.dashboard_list import DashboardList

        return {
            "dashboard_lists": ([DashboardList],),
        }

    attribute_map = {
        "dashboard_lists": "dashboard_lists",
    }

    def __init__(self_, dashboard_lists: Union[List[DashboardList], UnsetType] = unset, **kwargs):
        """
        Information on your dashboard lists.

        :param dashboard_lists: List of all your dashboard lists.
        :type dashboard_lists: [DashboardList], optional
        """
        if dashboard_lists is not unset:
            kwargs["dashboard_lists"] = dashboard_lists
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.timeseries_background_type import TimeseriesBackgroundType
    from datadog_api_client.v1.model.widget_axis import WidgetAxis


class TimeseriesBackground(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.timeseries_background_type import TimeseriesBackgroundType
        from datadog_api_client.v1.model.widget_axis import WidgetAxis

        return {
            "type": (TimeseriesBackgroundType,),
            "yaxis": (WidgetAxis,),
        }

    attribute_map = {
        "type": "type",
        "yaxis": "yaxis",
    }

    def __init__(self_, type: TimeseriesBackgroundType, yaxis: Union[WidgetAxis, UnsetType] = unset, **kwargs):
        """
        Set a timeseries on the widget background.

        :param type: Timeseries is made using an area or bars.
        :type type: TimeseriesBackgroundType

        :param yaxis: Axis controls for the widget.
        :type yaxis: WidgetAxis, optional
        """
        if yaxis is not unset:
            kwargs["yaxis"] = yaxis
        super().__init__(kwargs)

        self_.type = type



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class SLOBulkDeleteResponseData(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "deleted": ([str],),
            "updated": ([str],),
        }

    attribute_map = {
        "deleted": "deleted",
        "updated": "updated",
    }

    def __init__(
        self_, deleted: Union[List[str], UnsetType] = unset, updated: Union[List[str], UnsetType] = unset, **kwargs
    ):
        """
        An array of service level objective objects.

        :param deleted: An array of service level objective object IDs that indicates
            which objects that were completely deleted.
        :type deleted: [str], optional

        :param updated: An array of service level objective object IDs that indicates
            which objects that were modified (objects for which at least one
            threshold was deleted, but that were not completely deleted).
        :type updated: [str], optional
        """
        if deleted is not unset:
            kwargs["deleted"] = deleted
        if updated is not unset:
            kwargs["updated"] = updated
        super().__init__(kwargs)



"""
Get hourly usage for Logs by Index returns "OK" response
"""

from datetime import datetime
from dateutil.relativedelta import relativedelta
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.usage_metering_api import UsageMeteringApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = UsageMeteringApi(api_client)
    response = api_instance.get_usage_logs_by_index(
        start_hr=(datetime.now() + relativedelta(days=-5)),
        end_hr=(datetime.now() + relativedelta(days=-3)),
    )

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Any, Dict, Union

from datadog_api_client.api_client import ApiClient, Endpoint as _Endpoint
from datadog_api_client.model_utils import (
    UnsetType,
    unset,
)
from datadog_api_client.v2.model.incident_teams_response import IncidentTeamsResponse
from datadog_api_client.v2.model.incident_related_object import IncidentRelatedObject
from datadog_api_client.v2.model.incident_team_response import IncidentTeamResponse
from datadog_api_client.v2.model.incident_team_create_request import IncidentTeamCreateRequest
from datadog_api_client.v2.model.incident_team_update_request import IncidentTeamUpdateRequest


class IncidentTeamsApi:
    """
    Create, update, delete and retrieve teams which can be associated with incidents.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self._create_incident_team_endpoint = _Endpoint(
            settings={
                "response_type": (IncidentTeamResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/teams",
                "operation_id": "create_incident_team",
                "http_method": "POST",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (IncidentTeamCreateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._delete_incident_team_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/teams/{team_id}",
                "operation_id": "delete_incident_team",
                "http_method": "DELETE",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "team_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "team_id",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["*/*"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._get_incident_team_endpoint = _Endpoint(
            settings={
                "response_type": (IncidentTeamResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/teams/{team_id}",
                "operation_id": "get_incident_team",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "team_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "team_id",
                    "location": "path",
                },
                "include": {
                    "openapi_types": (IncidentRelatedObject,),
                    "attribute": "include",
                    "location": "query",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_incident_teams_endpoint = _Endpoint(
            settings={
                "response_type": (IncidentTeamsResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/teams",
                "operation_id": "list_incident_teams",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "include": {
                    "openapi_types": (IncidentRelatedObject,),
                    "attribute": "include",
                    "location": "query",
                },
                "page_size": {
                    "openapi_types": (int,),
                    "attribute": "page[size]",
                    "location": "query",
                },
                "page_offset": {
                    "openapi_types": (int,),
                    "attribute": "page[offset]",
                    "location": "query",
                },
                "filter": {
                    "openapi_types": (str,),
                    "attribute": "filter",
                    "location": "query",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._update_incident_team_endpoint = _Endpoint(
            settings={
                "response_type": (IncidentTeamResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/teams/{team_id}",
                "operation_id": "update_incident_team",
                "http_method": "PATCH",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "team_id": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "team_id",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (IncidentTeamUpdateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

    def create_incident_team(
        self,
        body: IncidentTeamCreateRequest,
    ) -> IncidentTeamResponse:
        """Create a new incident team.

        Creates a new incident team.

        :param body: Incident Team Payload.
        :type body: IncidentTeamCreateRequest
        :rtype: IncidentTeamResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._create_incident_team_endpoint.call_with_http_info(**kwargs)

    def delete_incident_team(
        self,
        team_id: str,
    ) -> None:
        """Delete an existing incident team.

        Deletes an existing incident team.

        :param team_id: The ID of the incident team.
        :type team_id: str
        :rtype: None
        """
        kwargs: Dict[str, Any] = {}
        kwargs["team_id"] = team_id

        return self._delete_incident_team_endpoint.call_with_http_info(**kwargs)

    def get_incident_team(
        self,
        team_id: str,
        *,
        include: Union[IncidentRelatedObject, UnsetType] = unset,
    ) -> IncidentTeamResponse:
        """Get details of an incident team.

        Get details of an incident team. If the ``include[users]`` query parameter is provided,
        the included attribute will contain the users related to these incident teams.

        :param team_id: The ID of the incident team.
        :type team_id: str
        :param include: Specifies which types of related objects should be included in the response.
        :type include: IncidentRelatedObject, optional
        :rtype: IncidentTeamResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["team_id"] = team_id

        if include is not unset:
            kwargs["include"] = include

        return self._get_incident_team_endpoint.call_with_http_info(**kwargs)

    def list_incident_teams(
        self,
        *,
        include: Union[IncidentRelatedObject, UnsetType] = unset,
        page_size: Union[int, UnsetType] = unset,
        page_offset: Union[int, UnsetType] = unset,
        filter: Union[str, UnsetType] = unset,
    ) -> IncidentTeamsResponse:
        """Get a list of all incident teams.

        Get all incident teams for the requesting user's organization. If the ``include[users]`` query parameter is provided, the included attribute will contain the users related to these incident teams.

        :param include: Specifies which types of related objects should be included in the response.
        :type include: IncidentRelatedObject, optional
        :param page_size: Size for a given page. The maximum allowed value is 5000.
        :type page_size: int, optional
        :param page_offset: Specific offset to use as the beginning of the returned page.
        :type page_offset: int, optional
        :param filter: A search query that filters teams by name.
        :type filter: str, optional
        :rtype: IncidentTeamsResponse
        """
        kwargs: Dict[str, Any] = {}
        if include is not unset:
            kwargs["include"] = include

        if page_size is not unset:
            kwargs["page_size"] = page_size

        if page_offset is not unset:
            kwargs["page_offset"] = page_offset

        if filter is not unset:
            kwargs["filter"] = filter

        return self._list_incident_teams_endpoint.call_with_http_info(**kwargs)

    def update_incident_team(
        self,
        team_id: str,
        body: IncidentTeamUpdateRequest,
    ) -> IncidentTeamResponse:
        """Update an existing incident team.

        Updates an existing incident team. Only provide the attributes which should be updated as this request is a partial update.

        :param team_id: The ID of the incident team.
        :type team_id: str
        :param body: Incident Team Payload.
        :type body: IncidentTeamUpdateRequest
        :rtype: IncidentTeamResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["team_id"] = team_id

        kwargs["body"] = body

        return self._update_incident_team_endpoint.call_with_http_info(**kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.widget_comparator import WidgetComparator
    from datadog_api_client.v1.model.widget_palette import WidgetPalette


class WidgetConditionalFormat(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.widget_comparator import WidgetComparator
        from datadog_api_client.v1.model.widget_palette import WidgetPalette

        return {
            "comparator": (WidgetComparator,),
            "custom_bg_color": (str,),
            "custom_fg_color": (str,),
            "hide_value": (bool,),
            "image_url": (str,),
            "metric": (str,),
            "palette": (WidgetPalette,),
            "timeframe": (str,),
            "value": (float,),
        }

    attribute_map = {
        "comparator": "comparator",
        "custom_bg_color": "custom_bg_color",
        "custom_fg_color": "custom_fg_color",
        "hide_value": "hide_value",
        "image_url": "image_url",
        "metric": "metric",
        "palette": "palette",
        "timeframe": "timeframe",
        "value": "value",
    }

    def __init__(
        self_,
        comparator: WidgetComparator,
        palette: WidgetPalette,
        value: float,
        custom_bg_color: Union[str, UnsetType] = unset,
        custom_fg_color: Union[str, UnsetType] = unset,
        hide_value: Union[bool, UnsetType] = unset,
        image_url: Union[str, UnsetType] = unset,
        metric: Union[str, UnsetType] = unset,
        timeframe: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        Define a conditional format for the widget.

        :param comparator: Comparator to apply.
        :type comparator: WidgetComparator

        :param custom_bg_color: Color palette to apply to the background, same values available as palette.
        :type custom_bg_color: str, optional

        :param custom_fg_color: Color palette to apply to the foreground, same values available as palette.
        :type custom_fg_color: str, optional

        :param hide_value: True hides values.
        :type hide_value: bool, optional

        :param image_url: Displays an image as the background.
        :type image_url: str, optional

        :param metric: Metric from the request to correlate this conditional format with.
        :type metric: str, optional

        :param palette: Color palette to apply.
        :type palette: WidgetPalette

        :param timeframe: Defines the displayed timeframe.
        :type timeframe: str, optional

        :param value: Value for the comparator.
        :type value: float
        """
        if custom_bg_color is not unset:
            kwargs["custom_bg_color"] = custom_bg_color
        if custom_fg_color is not unset:
            kwargs["custom_fg_color"] = custom_fg_color
        if hide_value is not unset:
            kwargs["hide_value"] = hide_value
        if image_url is not unset:
            kwargs["image_url"] = image_url
        if metric is not unset:
            kwargs["metric"] = metric
        if timeframe is not unset:
            kwargs["timeframe"] = timeframe
        super().__init__(kwargs)

        self_.comparator = comparator
        self_.palette = palette
        self_.value = value



"""
Edit an app key owned by this service account returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.service_accounts_api import ServiceAccountsApi
from datadog_api_client.v2.model.application_key_update_attributes import ApplicationKeyUpdateAttributes
from datadog_api_client.v2.model.application_key_update_data import ApplicationKeyUpdateData
from datadog_api_client.v2.model.application_key_update_request import ApplicationKeyUpdateRequest
from datadog_api_client.v2.model.application_keys_type import ApplicationKeysType

# there is a valid "service_account_user" in the system
SERVICE_ACCOUNT_USER_DATA_ID = environ["SERVICE_ACCOUNT_USER_DATA_ID"]

# there is a valid "service_account_application_key" for "service_account_user"
SERVICE_ACCOUNT_APPLICATION_KEY_DATA_ATTRIBUTES_NAME = environ["SERVICE_ACCOUNT_APPLICATION_KEY_DATA_ATTRIBUTES_NAME"]
SERVICE_ACCOUNT_APPLICATION_KEY_DATA_ID = environ["SERVICE_ACCOUNT_APPLICATION_KEY_DATA_ID"]

body = ApplicationKeyUpdateRequest(
    data=ApplicationKeyUpdateData(
        id=SERVICE_ACCOUNT_APPLICATION_KEY_DATA_ID,
        type=ApplicationKeysType.APPLICATION_KEYS,
        attributes=ApplicationKeyUpdateAttributes(
            name="Application Key for managing dashboards-updated",
        ),
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = ServiceAccountsApi(api_client)
    response = api_instance.update_service_account_application_key(
        service_account_id=SERVICE_ACCOUNT_USER_DATA_ID, app_key_id=SERVICE_ACCOUNT_APPLICATION_KEY_DATA_ID, body=body
    )

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.widget_service_summary_display_format import WidgetServiceSummaryDisplayFormat
    from datadog_api_client.v1.model.widget_size_format import WidgetSizeFormat
    from datadog_api_client.v1.model.widget_time import WidgetTime
    from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign
    from datadog_api_client.v1.model.service_summary_widget_definition_type import ServiceSummaryWidgetDefinitionType


class ServiceSummaryWidgetDefinition(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.widget_service_summary_display_format import WidgetServiceSummaryDisplayFormat
        from datadog_api_client.v1.model.widget_size_format import WidgetSizeFormat
        from datadog_api_client.v1.model.widget_time import WidgetTime
        from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign
        from datadog_api_client.v1.model.service_summary_widget_definition_type import (
            ServiceSummaryWidgetDefinitionType,
        )

        return {
            "display_format": (WidgetServiceSummaryDisplayFormat,),
            "env": (str,),
            "service": (str,),
            "show_breakdown": (bool,),
            "show_distribution": (bool,),
            "show_errors": (bool,),
            "show_hits": (bool,),
            "show_latency": (bool,),
            "show_resource_list": (bool,),
            "size_format": (WidgetSizeFormat,),
            "span_name": (str,),
            "time": (WidgetTime,),
            "title": (str,),
            "title_align": (WidgetTextAlign,),
            "title_size": (str,),
            "type": (ServiceSummaryWidgetDefinitionType,),
        }

    attribute_map = {
        "display_format": "display_format",
        "env": "env",
        "service": "service",
        "show_breakdown": "show_breakdown",
        "show_distribution": "show_distribution",
        "show_errors": "show_errors",
        "show_hits": "show_hits",
        "show_latency": "show_latency",
        "show_resource_list": "show_resource_list",
        "size_format": "size_format",
        "span_name": "span_name",
        "time": "time",
        "title": "title",
        "title_align": "title_align",
        "title_size": "title_size",
        "type": "type",
    }

    def __init__(
        self_,
        env: str,
        service: str,
        span_name: str,
        type: ServiceSummaryWidgetDefinitionType,
        display_format: Union[WidgetServiceSummaryDisplayFormat, UnsetType] = unset,
        show_breakdown: Union[bool, UnsetType] = unset,
        show_distribution: Union[bool, UnsetType] = unset,
        show_errors: Union[bool, UnsetType] = unset,
        show_hits: Union[bool, UnsetType] = unset,
        show_latency: Union[bool, UnsetType] = unset,
        show_resource_list: Union[bool, UnsetType] = unset,
        size_format: Union[WidgetSizeFormat, UnsetType] = unset,
        time: Union[WidgetTime, UnsetType] = unset,
        title: Union[str, UnsetType] = unset,
        title_align: Union[WidgetTextAlign, UnsetType] = unset,
        title_size: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        The service summary displays the graphs of a chosen service in your screenboard. Only available on FREE layout dashboards.

        :param display_format: Number of columns to display.
        :type display_format: WidgetServiceSummaryDisplayFormat, optional

        :param env: APM environment.
        :type env: str

        :param service: APM service.
        :type service: str

        :param show_breakdown: Whether to show the latency breakdown or not.
        :type show_breakdown: bool, optional

        :param show_distribution: Whether to show the latency distribution or not.
        :type show_distribution: bool, optional

        :param show_errors: Whether to show the error metrics or not.
        :type show_errors: bool, optional

        :param show_hits: Whether to show the hits metrics or not.
        :type show_hits: bool, optional

        :param show_latency: Whether to show the latency metrics or not.
        :type show_latency: bool, optional

        :param show_resource_list: Whether to show the resource list or not.
        :type show_resource_list: bool, optional

        :param size_format: Size of the widget.
        :type size_format: WidgetSizeFormat, optional

        :param span_name: APM span name.
        :type span_name: str

        :param time: Time setting for the widget.
        :type time: WidgetTime, optional

        :param title: Title of the widget.
        :type title: str, optional

        :param title_align: How to align the text on the widget.
        :type title_align: WidgetTextAlign, optional

        :param title_size: Size of the title.
        :type title_size: str, optional

        :param type: Type of the service summary widget.
        :type type: ServiceSummaryWidgetDefinitionType
        """
        if display_format is not unset:
            kwargs["display_format"] = display_format
        if show_breakdown is not unset:
            kwargs["show_breakdown"] = show_breakdown
        if show_distribution is not unset:
            kwargs["show_distribution"] = show_distribution
        if show_errors is not unset:
            kwargs["show_errors"] = show_errors
        if show_hits is not unset:
            kwargs["show_hits"] = show_hits
        if show_latency is not unset:
            kwargs["show_latency"] = show_latency
        if show_resource_list is not unset:
            kwargs["show_resource_list"] = show_resource_list
        if size_format is not unset:
            kwargs["size_format"] = size_format
        if time is not unset:
            kwargs["time"] = time
        if title is not unset:
            kwargs["title"] = title
        if title_align is not unset:
            kwargs["title_align"] = title_align
        if title_size is not unset:
            kwargs["title_size"] = title_size
        super().__init__(kwargs)

        self_.env = env
        self_.service = service
        self_.span_name = span_name
        self_.type = type



"""
Client is resilient to enum and oneOf deserialization errors
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.synthetics_api import SyntheticsApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = SyntheticsApi(api_client)
    response = api_instance.list_tests()

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class TableWidgetHasSearchBar(ModelSimple):
    """
    Controls the display of the search bar.

    :param value: Must be one of ["always", "never", "auto"].
    :type value: str
    """

    allowed_values = {
        "always",
        "never",
        "auto",
    }
    ALWAYS: ClassVar["TableWidgetHasSearchBar"]
    NEVER: ClassVar["TableWidgetHasSearchBar"]
    AUTO: ClassVar["TableWidgetHasSearchBar"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


TableWidgetHasSearchBar.ALWAYS = TableWidgetHasSearchBar("always")
TableWidgetHasSearchBar.NEVER = TableWidgetHasSearchBar("never")
TableWidgetHasSearchBar.AUTO = TableWidgetHasSearchBar("auto")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    none_type,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.unit import Unit


class ScalarColumn(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.unit import Unit

        return {
            "unit": ([Unit, none_type],),
            "values": ([float],),
        }

    attribute_map = {
        "unit": "unit",
        "values": "values",
    }

    def __init__(
        self_, unit: Union[List[Unit], UnsetType] = unset, values: Union[List[float], UnsetType] = unset, **kwargs
    ):
        """
        A single column in a scalar query response.

        :param unit: List of units.
        :type unit: [Unit, none_type], optional

        :param values: Array of values for each group-by combination that results from one formula or query.
        :type values: [float], optional
        """
        if unit is not unset:
            kwargs["unit"] = unit
        if values is not unset:
            kwargs["values"] = values
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


class LogQueryDefinitionSearch(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "query": (str,),
        }

    attribute_map = {
        "query": "query",
    }

    def __init__(self_, query: str, **kwargs):
        """
        The query being made on the logs.

        :param query: Search value to apply.
        :type query: str
        """
        super().__init__(kwargs)

        self_.query = query



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.security_monitoring_signal_incidents_update_attributes import (
        SecurityMonitoringSignalIncidentsUpdateAttributes,
    )


class SecurityMonitoringSignalIncidentsUpdateData(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.security_monitoring_signal_incidents_update_attributes import (
            SecurityMonitoringSignalIncidentsUpdateAttributes,
        )

        return {
            "attributes": (SecurityMonitoringSignalIncidentsUpdateAttributes,),
        }

    attribute_map = {
        "attributes": "attributes",
    }

    def __init__(self_, attributes: SecurityMonitoringSignalIncidentsUpdateAttributes, **kwargs):
        """
        Data containing the patch for changing the related incidents of a signal.

        :param attributes: Attributes describing the new list of related signals for a security signal.
        :type attributes: SecurityMonitoringSignalIncidentsUpdateAttributes
        """
        super().__init__(kwargs)

        self_.attributes = attributes



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class ListStreamColumnWidth(ModelSimple):
    """
    Widget column width.

    :param value: Must be one of ["auto", "compact", "full"].
    :type value: str
    """

    allowed_values = {
        "auto",
        "compact",
        "full",
    }
    AUTO: ClassVar["ListStreamColumnWidth"]
    COMPACT: ClassVar["ListStreamColumnWidth"]
    FULL: ClassVar["ListStreamColumnWidth"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


ListStreamColumnWidth.AUTO = ListStreamColumnWidth("auto")
ListStreamColumnWidth.COMPACT = ListStreamColumnWidth("compact")
ListStreamColumnWidth.FULL = ListStreamColumnWidth("full")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.monitor_formula_and_function_event_query_group_by_sort import (
        MonitorFormulaAndFunctionEventQueryGroupBySort,
    )


class MonitorFormulaAndFunctionEventQueryGroupBy(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.monitor_formula_and_function_event_query_group_by_sort import (
            MonitorFormulaAndFunctionEventQueryGroupBySort,
        )

        return {
            "facet": (str,),
            "limit": (int,),
            "sort": (MonitorFormulaAndFunctionEventQueryGroupBySort,),
        }

    attribute_map = {
        "facet": "facet",
        "limit": "limit",
        "sort": "sort",
    }

    def __init__(
        self_,
        facet: str,
        limit: Union[int, UnsetType] = unset,
        sort: Union[MonitorFormulaAndFunctionEventQueryGroupBySort, UnsetType] = unset,
        **kwargs,
    ):
        """
        List of objects used to group by.

        :param facet: Event facet.
        :type facet: str

        :param limit: Number of groups to return.
        :type limit: int, optional

        :param sort: Options for sorting group by results.
        :type sort: MonitorFormulaAndFunctionEventQueryGroupBySort, optional
        """
        if limit is not unset:
            kwargs["limit"] = limit
        if sort is not unset:
            kwargs["sort"] = sort
        super().__init__(kwargs)

        self_.facet = facet



"""
Edit an application key for this service account returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.service_accounts_api import ServiceAccountsApi
from datadog_api_client.v2.model.application_key_update_attributes import ApplicationKeyUpdateAttributes
from datadog_api_client.v2.model.application_key_update_data import ApplicationKeyUpdateData
from datadog_api_client.v2.model.application_key_update_request import ApplicationKeyUpdateRequest
from datadog_api_client.v2.model.application_keys_type import ApplicationKeysType

body = ApplicationKeyUpdateRequest(
    data=ApplicationKeyUpdateData(
        attributes=ApplicationKeyUpdateAttributes(
            name="Application Key for managing dashboards",
            scopes=[
                "dashboards_read",
                "dashboards_write",
                "dashboards_public_share",
            ],
        ),
        id="00112233-4455-6677-8899-aabbccddeeff",
        type=ApplicationKeysType.APPLICATION_KEYS,
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = ServiceAccountsApi(api_client)
    response = api_instance.update_service_account_application_key(
        service_account_id="00000000-0000-1234-0000-000000000000", app_key_id="app_key_id", body=body
    )

    print(response)
